@@ -12,17 +12,19 @@ from sqlobject import SQLObjectNotFound
 # we need string.punctuation
 import string
 # pylint: enable-msg=W0402
-from sutekh.core.SutekhObjects import AbstractCard, IPhysicalCard, IExpansion
-from sutekh.core.Filters import CardNameFilter
+from sutekh.core.SutekhObjects import IPhysicalCard, IExpansion, IAbstractCard
+from sutekh.core.Filters import CardNameFilter, FilterAndBox, \
+        make_illegal_filter
+
 
 # pylint: disable-msg=R0922
 # We inherit from these classes elsewhere
-
 class LookupFailed(Exception):
     """Raised when an AbstractCard lookup fails completed.
        """
     pass
 
+
 class AbstractCardLookup(object):
     """Base class for objects which translate card names into abstract card
        objects.
@@ -38,6 +40,7 @@ class AbstractCardLookup(object):
            """
         raise NotImplementedError
 
+
 class PhysicalCardLookup(object):
     """Base class for objects which translate card and expansion names
        into physical card objects
@@ -63,6 +66,7 @@ class PhysicalCardLookup(object):
            """
         raise NotImplementedError
 
+
 class ExpansionLookup(object):
     """Base class for objects which translate expansion names into expansion
        objects
@@ -78,16 +82,14 @@ class ExpansionLookup(object):
            """
         raise NotImplementedError
 
+
 class SimpleLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
     """A really straightforward lookup of AbstractCards and PhysicalCards.
 
        The default when we don't have a more cunning plan.
        """
 
-    # pylint: disable-msg=W0613
-    # sInfo is required by the function signature, but not used, since we
-    # don't actually interact with the user
-    def lookup(self, aNames, sInfo):
+    def lookup(self, aNames, _sInfo):
         """A lookup method that excludes unknown cards."""
         aCards = []
         for sName in aNames:
@@ -95,8 +97,7 @@ class SimpleLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
                 # pylint: disable-msg=E1101
                 # SQLObject confuses pylint
                 try:
-                    oAbs = AbstractCard.byCanonicalName(
-                            sName.encode('utf8').lower())
+                    oAbs = IAbstractCard(sName)
                     aCards.append(oAbs)
                 except SQLObjectNotFound:
                     aCards.append(None)
@@ -104,7 +105,7 @@ class SimpleLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
                 aCards.append(None)
         return aCards
 
-    def physical_lookup(self, dCardExpansions, dNameCards, dNameExps, sInfo):
+    def physical_lookup(self, dCardExpansions, dNameCards, dNameExps, _sInfo):
         """Lookup cards in the physical card set, excluding unknown cards."""
         aCards = []
         for sName in dCardExpansions:
@@ -116,14 +117,15 @@ class SimpleLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
                     try:
                         iCnt = dCardExpansions[sName][sExpansionName]
                         oExpansion = dNameExps[sExpansionName]
-                        aCards.extend([IPhysicalCard((oAbs, oExpansion))]*iCnt)
+                        aCards.extend(
+                                [IPhysicalCard((oAbs, oExpansion))] * iCnt)
                     except SQLObjectNotFound:
                         # This card is missing from the PhysicalCard list, so
                         # skipped
                         pass
         return aCards
 
-    def expansion_lookup(self, aExpansionNames, sInfo):
+    def expansion_lookup(self, aExpansionNames, _sInfo):
         """Lookup for expansion names, excluding unknown expansions."""
         aExps = []
         for sExp in aExpansionNames:
@@ -163,9 +165,8 @@ def best_guess_filter(sName):
     sFilterString = sFilterString.replace(' ', '%')
     # Add % on outside
     sFilterString = '%' + sFilterString + '%'
-    return CardNameFilter(sFilterString)
+    oLegalFilter = make_illegal_filter()
+    return FilterAndBox([CardNameFilter(sFilterString), oLegalFilter])
 
 
-# pylint: disable-msg=C0103
-# DEFAULT_LOOKUP name is OK here
 DEFAULT_LOOKUP = SimpleLookup()
