@@ -10,33 +10,45 @@
    final Filter Object is constructed
    """
 
+# pylint: disable-msg=E0611
+# pylint 0.18 misses ply parts
 import ply.lex as lex
 import ply.yacc as yacc
-from sutekh.core.Filters import aParserFilters, FilterNot, FilterAndBox, \
+# pylint: enable-msg=E0611
+from sutekh.core.Filters import PARSER_FILTERS, FilterNot, FilterAndBox, \
         FilterOrBox
 
-aEntryFilters = [x.keyword for x in aParserFilters if hasattr(x,'istextentry')
-        and x.istextentry]
-aWithFilters = [x.keyword for x in aParserFilters if hasattr(x,'iswithfilter')
-        and x.iswithfilter]
-aFromFilters = [x.keyword for x in aParserFilters if hasattr(x,'isfromfilter')
-        and x.isfromfilter]
-aListFilters = [x.keyword for x in aParserFilters if hasattr(x,'islistfilter')
-        and x.islistfilter]
 
+ENTRY_FILTERS = set([x.keyword for x in PARSER_FILTERS
+    if hasattr(x, 'istextentry') and x.istextentry])
+WITH_FILTERS = set([x.keyword for x in PARSER_FILTERS
+    if hasattr(x, 'iswithfilter') and x.iswithfilter])
+FROM_FILTERS = set([x.keyword for x in PARSER_FILTERS
+    if hasattr(x, 'isfromfilter') and x.isfromfilter])
+LIST_FILTERS = set([x.keyword for x in PARSER_FILTERS
+    if hasattr(x, 'islistfilter') and x.islistfilter])
+
+
+# Misc utility functions
 def get_filter_type(sKeyword):
     """Get the actual filter object from the type string"""
     # pylint: disable-msg=W0621
-    return [x for x in aParserFilters if x.keyword == sKeyword][0]
+    return [x for x in PARSER_FILTERS if x.keyword == sKeyword][0]
+
+
+def get_filters_for_type(sFilterType):
+    """Get the filters associated with this filter type."""
+    return [oFilterType for oFilterType in PARSER_FILTERS
+            if sFilterType in oFilterType.types]
+
 
 # We define an object for the lex parser
-
 # pylint is never going to like the naming conventions here,
 # which are based on ply examples
 class ParseFilterDefinitions(object):
-    """Provides the lexxer used by PLY"""
+    """Provides the lexer used by PLY"""
     # pylint: disable-msg=C0103, R0201
-    aKeywords = [x.keyword for x in aParserFilters]
+    aKeywords = set([x.keyword for x in PARSER_FILTERS])
 
     tokens = (
             'NOT',
@@ -56,7 +68,7 @@ class ParseFilterDefinitions(object):
 
     t_AND = r'\&\&'
     t_OR = r'\|\|'
-    t_STRING = r'\".*?\"'
+    t_STRING = r'"(\\\\|\\\'|\\"|[^"\\])*?"|\'(\\\\|\\\'|\\"|[^\'\\])*?\''
     t_COMMA = r','
     t_IN = r'='
     t_LPAREN = r'\('
@@ -99,7 +111,7 @@ class ParseFilterDefinitions(object):
 
            Done outside of __init__, as PLY docs state this is required
            """
-        self.oLexer = lex.lex(object = self, **kwargs)
+        self.oLexer = lex.lex(object=self, **kwargs)
 
     def apply(self, sData):
         """Apply the lexer to the string sData"""
@@ -111,8 +123,8 @@ class ParseFilterDefinitions(object):
                 return aResults
             aResults.append(oToken)
 
-# Define a yacc parser to produce the abstract syntax tree
 
+# Define a yacc parser to produce the abstract syntax tree
 class FilterYaccParser(object):
     """Provide the parser used by PLY"""
     # pylint: disable-msg=C0103, R0201
@@ -222,8 +234,8 @@ class FilterYaccParser(object):
         else:
             raise ValueError("Invalid identifier: %s " % p.value)
 
-# Wrapper objects around the parser
 
+# Wrapper objects around the parser
 class FilterParser(object):
     """Entry point for filter parsing. Wraps Lexer and Parser Objects"""
     # pylint: disable-msg=R0903
@@ -241,7 +253,7 @@ class FilterParser(object):
         if not self._oGlobalParser:
             # yacc needs an initialised lexer
             self._oGlobalFilterParser = FilterYaccParser()
-            self._oGlobalParser = yacc.yacc(module = self._oGlobalFilterParser,
+            self._oGlobalParser = yacc.yacc(module=self._oGlobalFilterParser,
                                             debug=0,
                                             write_tables=0)
 
@@ -255,14 +267,40 @@ class FilterParser(object):
             oAST = self._oGlobalParser.parse(' ')
         return oAST
 
+
+# Helper functions for dealing with strings
+def escape(sData):
+    """Escape quotes and \\'s in the string"""
+    # escape \'s
+    sResult = sData.replace('\\', '\\\\')
+    # escape quotes
+    sResult = sResult.replace("'", "\\'")
+    sResult = sResult.replace('"', '\\"')
+    return sResult
+
+
+def unescape(sData):
+    """Unescape quotes and \\'s in the string.
+
+       should be the inverse of escape."""
+    # unescape quotes
+    # We assume that this is called on a escaped sring, after any
+    # surrounding quotes have been stripped off, so there are no unescaped
+    # quotes to worry about
+    sResult = sData.replace("\\'", "'")
+    sResult = sResult.replace('\\"', '"')
+    # unsecape \\
+    sResult = sResult.replace('\\\\', '\\')
+    return sResult
+
+
 # Object used by get_values representation
 # Should be made more robust
-
 class ValueObject(object):
     """Object to represent values extracted from the AST"""
 
     def __init__(self, oValue, oNode):
-        """Initialize ValueObject with oValue and AST node oNode"""
+        """Initialise ValueObject with oValue and AST node oNode"""
         self.oValue = oValue
         self.oNode = oNode
 
@@ -287,9 +325,9 @@ class ValueObject(object):
         """Is this node's value None?"""
         return self.oValue is None
 
+
 # AST object (formulation inspired by Simon Cross's example, and notes
 # from the ply documentation)
-
 class AstBaseNode(object):
     """Basic node class for the AST. Other nodes inherit from this"""
 
@@ -302,7 +340,7 @@ class AstBaseNode(object):
 
            Useful for debugging
            """
-        sAttrs = '(' + ",".join([ str(oValue) for sKey, oValue \
+        sAttrs = '(' + ",".join([str(oValue) for sKey, oValue \
                 in self.__dict__.items() if not sKey.startswith("_") and \
                 sKey != "aChildren" and oValue not in self.aChildren]) + ")"
         sOutput = self.__class__.__name__ + sAttrs
@@ -328,6 +366,7 @@ class AstBaseNode(object):
         """Get type information of the associated filter"""
         pass
 
+
 class FilterNode(AstBaseNode):
     """Represents a Filter in the AST"""
 
@@ -341,7 +380,7 @@ class FilterNode(AstBaseNode):
         return self.oExpression
 
     def get_values(self):
-        """Get ilter values"""
+        """Get filter values"""
         if self.oExpression:
             return self.oExpression.get_values()
         else:
@@ -355,7 +394,7 @@ class FilterNode(AstBaseNode):
             return None
 
     def get_invalid_values(self):
-        """Get values that are invlaid for this filter"""
+        """Get values that are invalid for this filter"""
         if self.oExpression:
             return self.oExpression.get_invalid_values()
         else:
@@ -368,12 +407,14 @@ class FilterNode(AstBaseNode):
         else:
             return None
 
+
 class OperatorNode(AstBaseNode):
     """Base class for nodes involving operators"""
     pass
 
+
 class TermNode(AstBaseNode):
-    """Node to represnt values in the AST"""
+    """Node to represent values in the AST"""
 
     def get_type(self):
         """Return type.
@@ -383,6 +424,7 @@ class TermNode(AstBaseNode):
            """
         return None
 
+
 class StringNode(TermNode):
     """String value in the AST"""
 
@@ -393,8 +435,12 @@ class StringNode(TermNode):
         # Strip quotes off strings
         if sValue[0] == '"' and sValue[-1] == '"':
             self.sValue = sValue[1:-1]
+        elif sValue[0] == "'" and sValue[-1] == "'":
+            self.sValue = sValue[1:-1]
         else:
             self.sValue = sValue
+        # Unescape quotes and \'s if needed
+        self.sValue = unescape(self.sValue)
 
     def get_values(self):
         """Return the ValueObject holding the string"""
@@ -404,11 +450,12 @@ class StringNode(TermNode):
         """The filter is the string, as a list"""
         return [self.sValue]
 
+
 class IdNode(TermNode):
     """$foo variable identifier in the AST"""
 
     def __init__(self, sValue):
-        """Initiliase IdNode. sValue == name of variable"""
+        """Initialise IdNode. sValue == name of variable"""
         super(IdNode, self).__init__([sValue])
         self.oValue = sValue
 
@@ -420,6 +467,7 @@ class IdNode(TermNode):
         """IdNode has no filter"""
         return None
 
+
 class FilterPartNode(OperatorNode):
     """A Filter = $X expression in the AST"""
 
@@ -433,14 +481,18 @@ class FilterPartNode(OperatorNode):
         """Variable name associated with the filter"""
         return self.sVariableName
 
+    def get_description(self):
+        """Get the description for the associated filter object"""
+        return get_filter_type(self.sFilterName).description
+
     def get_values(self):
         """List of ValueObjects describing the filter and its associated
            values."""
-        if self.sFilterName in aEntryFilters:
+        if self.sFilterName in ENTRY_FILTERS:
             aResults = \
                     [ValueObject(get_filter_type(self.sFilterName).description
                         + ' includes', self)]
-        elif self.sFilterName in aListFilters:
+        elif self.sFilterName in LIST_FILTERS:
             aResults = \
                     [ValueObject(get_filter_type(self.sFilterName).description
                         + ' in', self)]
@@ -449,22 +501,19 @@ class FilterPartNode(OperatorNode):
             # values to return
             return [ValueObject(get_filter_type(self.sFilterName).description,
                 self), ValueObject(None, self)]
-        if self.aFilterValues is None:
-            aVals = get_filter_type(self.sFilterName).get_values()
-            # Want a list within ValueObject for the GUI stuff to work
-            # '' case for Entry boxes works as well
-            aResults.append(ValueObject(aVals, self))
-        else:
-            aResults.extend(self.aFilterValues.get_values())
+        # Want a list within ValueObject for the GUI stuff to work
+        # '' case for Entry boxes works as well
+        aVals = get_filter_type(self.sFilterName).get_values()
+        aResults.append(ValueObject(aVals, self))
         return aResults
 
     def get_invalid_values(self):
         """List of illegal values associated with this filter"""
         aRes = []
-        if self.aFilterValues is None or self.sFilterName not in aListFilters:
+        if self.aFilterValues is None or self.sFilterName not in LIST_FILTERS:
             return None
         aCurVals = self.aFilterValues.get_values()
-        oTemp = get_filter_type(self.sFilterName)([]) # Create Instance
+        oTemp = get_filter_type(self.sFilterName)([])  # Create Instance
         aValidVals = oTemp.get_values()
         if isinstance(aValidVals[0], list) and len(aCurVals) == \
                 len(aValidVals):
@@ -480,7 +529,7 @@ class FilterPartNode(OperatorNode):
                     continue
                 if oVal.oValue not in aValidVals:
                     aRes.append(oVal.oValue)
-        if len(aRes)>0:
+        if len(aRes) > 0:
             return aRes
         else:
             return None
@@ -489,10 +538,14 @@ class FilterPartNode(OperatorNode):
         """Set values for this filter"""
         if self.aFilterValues is not None:
             raise RuntimeError("Filter values already set")
-        if self.sFilterName in aFromFilters:
-            sCommaList = ",".join(aVals[0])
-            sInternalFilter = self.sFilterName + '=' + sCommaList + 'from' + \
-                    aVals[1]
+        if not aVals:
+            # Skip empty values
+            return
+        elif self.sFilterName in FROM_FILTERS:
+            sCountList = ",".join(aVals[0])
+            sSetList = ",".join(aVals[1])
+            sInternalFilter = self.sFilterName + '=' + sCountList + 'from' + \
+                    sSetList
         else:
             sCommaList = ",".join(aVals)
             sInternalFilter = self.sFilterName + '=' + sCommaList
@@ -506,14 +559,14 @@ class FilterPartNode(OperatorNode):
 
     def get_filter(self):
         """Get Filter object for this Filter"""
-        if self.sFilterName in aEntryFilters or \
-                self.sFilterName in aListFilters:
+        if self.sFilterName in ENTRY_FILTERS or \
+                self.sFilterName in LIST_FILTERS:
             if self.aFilterValues is None:
                 return None
         cFilterType = get_filter_type(self.sFilterName)
         if self.aFilterValues:
             aValues = self.aFilterValues.get_filter()
-            if self.sFilterName in aEntryFilters:
+            if self.sFilterName in ENTRY_FILTERS:
                 # Filter takes a single string as input
                 # by construction, this is aValues[0]
                 oFilter = cFilterType(aValues[0])
@@ -528,6 +581,7 @@ class FilterPartNode(OperatorNode):
         """Get allowed types for this filter"""
         return get_filter_type(self.sFilterName).types
 
+
 class NotOpNode(OperatorNode):
     """AST node for NOT(X)"""
     def __init__(self, oSubExpression):
@@ -538,6 +592,13 @@ class NotOpNode(OperatorNode):
         """NOT(X)'s invalid values are the subfilter's invalid values"""
         return self.oSubExpression.get_invalid_values()
 
+    def get_description(self):
+        """NOT(X)'s description is the subfilter's description,
+           if applicable"""
+        if hasattr(self.oSubExpression, 'get_description'):
+            return self.oSubExpression.get_description()
+        return None
+
     def get_values(self):
         """NOT(X)'s values are the subfilter's values"""
         aResults = [ValueObject('NOT (', None)] + \
@@ -553,6 +614,7 @@ class NotOpNode(OperatorNode):
         """Get the filter type (same as subfilter)"""
         return self.oSubExpression.get_type()
 
+
 class BinOpNode(OperatorNode):
     """AST node for binary operations (AND, OR)"""
     def __init__(self, oLeft, oOp, oRight):
@@ -619,8 +681,9 @@ class BinOpNode(OperatorNode):
                 aRes.append(sType)
         return aRes
 
+
 class CommaNode(OperatorNode):
-    """AST node for comma seperator (Val1, Val2)"""
+    """AST node for comma separator (Val1, Val2)"""
     def __init__(self, oLeft, oOp, oRight):
         super(CommaNode, self).__init__([oLeft, oRight])
         self.oOp = oOp
@@ -645,6 +708,7 @@ class CommaNode(OperatorNode):
         # Syntax ensures , only in value lists, which have no type
         return None
 
+
 class WithNode(OperatorNode):
     """AST node for values of the form 'X with Y'"""
     def __init__(self, oLeft, oOp, oRight):
@@ -667,8 +731,9 @@ class WithNode(OperatorNode):
         # Syntax ensures with only in value lists, which have no type
         return None
 
+
 class FromNode(OperatorNode):
-    """AST node for values of the form 'X, Y from Z'"""
+    """AST node for values of the form 'X, Y from W, Z'"""
     def __init__(self, oLeft, oOp, oRight):
         super(FromNode, self).__init__([oLeft, oRight])
         self.oOp = oOp
@@ -677,12 +742,12 @@ class FromNode(OperatorNode):
 
     def get_values(self):
         """Get values"""
-        return [[self.oLeft.get_values()],
-            self.oRight.get_filter()[0]]
+        return [self.oLeft.get_values(),
+            self.oRight.get_values()]
 
     def get_filter(self):
         """Get filter expression"""
-        return [self.oLeft.get_filter(), self.oRight.get_filter()[0]]
+        return [self.oLeft.get_filter(), self.oRight.get_filter()]
 
     def get_type(self):
         """Get filter type - these are values, so always None"""
