@@ -5,6 +5,11 @@
 # Copyright 2006, 2007 Neil Muller <drnlmuller+sutekh@gmail.com>
 # GPL - see COPYING for details
 
+# pylint: disable-msg=C0111, C0321, C0302
+# C0111 - No point in docstrings for these classes, really
+# C0321 - Compactness of single line definitions is good here
+# C0302 - lots of lines as we want all these related definitions in one file
+
 """The database definitions and pyprotocols adaptors for Sutekh"""
 
 from sutekh.core.CachedRelatedJoin import CachedRelatedJoin, \
@@ -21,28 +26,61 @@ from protocols import advise, Interface
 
 # Interfaces
 
-# pylint: disable-msg=R0923, C0111, C0321
-# R0923 - PyProtocols magic will take care of the implementation
-# C0111 - No point in docstrings for these classes, really
-# C0321 - Compactness of single line definitions is good here
 
 class IAbstractCard(Interface): pass
+
+
 class IPhysicalCard(Interface): pass
+
+
 class IPhysicalCardSet(Interface): pass
+
+
 class IRarityPair(Interface): pass
+
+
 class IExpansion(Interface): pass
+
+
+class IExpansionName(Interface): pass
+
+
 class IRarity(Interface): pass
+
+
 class IDisciplinePair(Interface): pass
+
+
 class IDiscipline(Interface): pass
+
+
 class IClan(Interface): pass
+
+
 class ICardType(Interface): pass
+
+
 class ISect(Interface): pass
+
+
 class ITitle(Interface): pass
+
+
 class ICreed(Interface): pass
+
+
 class IVirtue(Interface): pass
+
+
 class IRuling(Interface): pass
 
-# pylint: enable-msg=R0923, C0111, C0321
+
+class IArtist(Interface): pass
+
+
+class IKeyword(Interface): pass
+
+# pylint: enable-msg=C0321
 # Table Objects
 
 # pylint: disable-msg=W0232, R0902, W0201, C0103
@@ -51,19 +89,27 @@ class IRuling(Interface): pass
 # W0201: We don't care about attributes defined outside init, by design
 # C0103: We use different naming conventions for the table columns
 
+# We try to avoid limiting the length of unicode columns but we have
+# to provide a length for alternate id columns and index columns.
+# For these we default to 512 characters.
+
+MAX_ID_LENGTH = 512
+
+
 class VersionTable(SQLObject):
     TableName = UnicodeCol(alternateID=True, length=50)
     Version = IntCol(default=None)
     tableversion = 1
 
+
 class AbstractCard(SQLObject):
     advise(instancesProvide=[IAbstractCard])
 
-    tableversion = 4
+    tableversion = 5
     sqlmeta.lazyUpdate = True
 
-    canonicalName = UnicodeCol(alternateID=True, length=50)
-    name = UnicodeCol(length=50)
+    canonicalName = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    name = UnicodeCol()
     text = UnicodeCol()
     group = IntCol(default=None, dbName='grp')
     capacity = IntCol(default=None)
@@ -72,8 +118,11 @@ class AbstractCard(SQLObject):
     costtype = EnumCol(enumValues=['pool', 'blood', 'conviction', None],
             default=None)
     level = EnumCol(enumValues=['advanced', None], default=None)
-    burnoption = BoolCol(default=False)
 
+    # Most of these names are singular when they should be plural
+    # since they refer to lists. We've decided to live with the
+    # inconsistency for old columns but do the right thing for new
+    # ones.
     discipline = CachedRelatedJoin('DisciplinePair',
             intermediateTable='abs_discipline_pair_map',
             createRelatedTable=False)
@@ -94,9 +143,14 @@ class AbstractCard(SQLObject):
             createRelatedTable=False)
     rulings = CachedRelatedJoin('Ruling', intermediateTable='abs_ruling_map',
             createRelatedTable=False)
+    artists = CachedRelatedJoin('Artist', intermediateTable='abs_artist_map',
+            createRelatedTable=False)
+    keywords = CachedRelatedJoin('Keyword',
+            intermediateTable='abs_keyword_map', createRelatedTable=False)
 
     physicalCards = MultipleJoin('PhysicalCard')
 
+
 class PhysicalCard(SQLObject):
     advise(instancesProvide=[IPhysicalCard])
 
@@ -108,12 +162,13 @@ class PhysicalCard(SQLObject):
     sets = RelatedJoin('PhysicalCardSet', intermediateTable='physical_map',
             createRelatedTable=False)
 
+
 class PhysicalCardSet(SQLObject):
     advise(instancesProvide=[IPhysicalCardSet])
 
-    tableversion = 5
-    name = UnicodeCol(alternateID=True, length=50)
-    author = UnicodeCol(length=50, default='')
+    tableversion = 6
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    author = UnicodeCol(default='')
     comment = UnicodeCol(default='')
     annotations = UnicodeCol(default='')
     inuse = BoolCol(default=False)
@@ -121,6 +176,7 @@ class PhysicalCardSet(SQLObject):
     cards = RelatedJoin('PhysicalCard', intermediateTable='physical_map',
             createRelatedTable=False)
 
+
 class RarityPair(SQLObject):
     advise(instancesProvide=[IRarityPair])
 
@@ -131,20 +187,23 @@ class RarityPair(SQLObject):
             intermediateTable='abs_rarity_pair_map', createRelatedTable=False)
     expansionRarityIndex = DatabaseIndex(expansion, rarity, unique=True)
 
+
 class Expansion(SQLObject):
     advise(instancesProvide=[IExpansion])
 
-    tableversion = 2
-    name = UnicodeCol(alternateID=True, length=20)
-    shortname = UnicodeCol(length=10, default=None)
+    tableversion = 3
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    shortname = UnicodeCol(default=None)
     pairs = MultipleJoin('RarityPair')
 
+
 class Rarity(SQLObject):
     advise(instancesProvide=[IRarity])
 
-    tableversion = 2
-    name = UnicodeCol(alternateID=True, length=20)
-    shortname = UnicodeCol(alternateID=True, length=20)
+    tableversion = 3
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    shortname = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+
 
 class DisciplinePair(SQLObject):
     advise(instancesProvide=[IDisciplinePair])
@@ -157,77 +216,109 @@ class DisciplinePair(SQLObject):
             intermediateTable='abs_discipline_pair_map',
             createRelatedTable=False)
 
+
 class Discipline(SQLObject):
     advise(instancesProvide=[IDiscipline])
 
-    tableversion = 2
-    name = UnicodeCol(alternateID=True, length=30)
-    fullname = UnicodeCol(length=30, default=None)
+    tableversion = 3
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    fullname = UnicodeCol(default=None)
     pairs = MultipleJoin('DisciplinePair')
 
+
 class Virtue(SQLObject):
     advise(instancesProvide=[IVirtue])
 
-    tableversion = 1
-    name = UnicodeCol(alternateID=True, length=30)
-    fullname = UnicodeCol(length=30, default=None)
+    tableversion = 2
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    fullname = UnicodeCol(default=None)
     cards = RelatedJoin('AbstractCard', intermediateTable='abs_virtue_map',
             createRelatedTable=False)
 
+
 class Creed(SQLObject):
     advise(instancesProvide=[ICreed])
 
-    tableversion = 1
-    name = UnicodeCol(alternateID=True, length=40)
-    shortname = UnicodeCol(length=10, default=None)
+    tableversion = 2
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    shortname = UnicodeCol(default=None)
     cards = RelatedJoin('AbstractCard', intermediateTable='abs_creed_map',
             createRelatedTable=False)
 
+
 class Clan(SQLObject):
     advise(instancesProvide=[IClan])
 
-    tableversion = 2
-    name = UnicodeCol(alternateID=True, length=40)
-    shortname = UnicodeCol(length=10, default=None)
+    tableversion = 3
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    shortname = UnicodeCol(default=None)
     cards = RelatedJoin('AbstractCard', intermediateTable='abs_clan_map',
             createRelatedTable=False)
 
+
 class CardType(SQLObject):
     advise(instancesProvide=[ICardType])
 
-    tableversion = 1
-    name = UnicodeCol(alternateID=True, length=50)
+    tableversion = 2
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
     cards = RelatedJoin('AbstractCard', intermediateTable='abs_type_map',
             createRelatedTable=False)
 
+
 class Sect(SQLObject):
     advise(instancesProvide=[ISect])
 
-    tableversion = 1
-    name = UnicodeCol(alternateID=True, length=50)
+    tableversion = 2
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
     cards = RelatedJoin('AbstractCard', intermediateTable='abs_sect_map',
             createRelatedTable=False)
 
+
 class Title(SQLObject):
     advise(instancesProvide=[ITitle])
 
-    tableversion = 1
-    name = UnicodeCol(alternateID=True, length=50)
+    tableversion = 2
+    name = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
     cards = RelatedJoin('AbstractCard', intermediateTable='abs_title_map',
             createRelatedTable=False)
 
+
 class Ruling(SQLObject):
     advise(instancesProvide=[IRuling])
 
-    tableversion = 1
-    text = UnicodeCol(alternateID=True, length=512)
-    code = UnicodeCol(length=50)
-    url = UnicodeCol(length=256, default=None)
+    tableversion = 2
+    text = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    code = UnicodeCol()
+    url = UnicodeCol(default=None)
     cards = RelatedJoin('AbstractCard', intermediateTable='abs_ruling_map',
             createRelatedTable=False)
 
+
+class Artist(SQLObject):
+    advise(instancesProvide=[IArtist])
+
+    tableversion = 1
+    canonicalName = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    name = UnicodeCol()
+    cards = RelatedJoin('AbstractCard', intermediateTable='abs_artist_map',
+            createRelatedTable=False)
+
+
+class Keyword(SQLObject):
+    advise(instancesProvide=[IKeyword])
+
+    # For sanity, avoid keywords with commas since this is the preferred
+    # character for separating lists of keywords when displaying them
+    # to a user in a compact way.
+
+    tableversion = 1
+    keyword = UnicodeCol(alternateID=True, length=MAX_ID_LENGTH)
+    cards = RelatedJoin('AbstractCard', intermediateTable='abs_keyword_map',
+            createRelatedTable=False)
+
 # Mapping Tables
 
+
 class MapPhysicalCardToPhysicalCardSet(SQLObject):
     class sqlmeta:
         table = 'physical_map'
@@ -241,6 +332,7 @@ class MapPhysicalCardToPhysicalCardSet(SQLObject):
     physicalCardSetIndex = DatabaseIndex(physicalCardSet, unique=False)
     jointIndex = DatabaseIndex(physicalCard, physicalCardSet, unique=False)
 
+
 class MapAbstractCardToRarityPair(SQLObject):
     class sqlmeta:
         table = 'abs_rarity_pair_map'
@@ -253,6 +345,7 @@ class MapAbstractCardToRarityPair(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     rarityPairIndex = DatabaseIndex(rarityPair, unique=False)
 
+
 class MapAbstractCardToRuling(SQLObject):
     class sqlmeta:
         table = 'abs_ruling_map'
@@ -265,6 +358,7 @@ class MapAbstractCardToRuling(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     rulingIndex = DatabaseIndex(ruling, unique=False)
 
+
 class MapAbstractCardToClan(SQLObject):
     class sqlmeta:
         table = 'abs_clan_map'
@@ -277,6 +371,7 @@ class MapAbstractCardToClan(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     clanIndex = DatabaseIndex(clan, unique=False)
 
+
 class MapAbstractCardToDisciplinePair(SQLObject):
     class sqlmeta:
         table = 'abs_discipline_pair_map'
@@ -289,6 +384,7 @@ class MapAbstractCardToDisciplinePair(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     disciplinePairIndex = DatabaseIndex(disciplinePair, unique=False)
 
+
 class MapAbstractCardToCardType(SQLObject):
     class sqlmeta:
         table = 'abs_type_map'
@@ -301,6 +397,7 @@ class MapAbstractCardToCardType(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     cardTypeIndex = DatabaseIndex(cardType, unique=False)
 
+
 class MapAbstractCardToSect(SQLObject):
     class sqlmeta:
         table = 'abs_sect_map'
@@ -313,6 +410,7 @@ class MapAbstractCardToSect(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     sectIndex = DatabaseIndex(sect, unique=False)
 
+
 class MapAbstractCardToTitle(SQLObject):
     class sqlmeta:
         table = 'abs_title_map'
@@ -325,6 +423,7 @@ class MapAbstractCardToTitle(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     titleIndex = DatabaseIndex(title, unique=False)
 
+
 class MapAbstractCardToCreed(SQLObject):
     class sqlmeta:
         table = 'abs_creed_map'
@@ -337,6 +436,7 @@ class MapAbstractCardToCreed(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     creedIndex = DatabaseIndex(creed, unique=False)
 
+
 class MapAbstractCardToVirtue(SQLObject):
     class sqlmeta:
         table = 'abs_virtue_map'
@@ -349,14 +449,41 @@ class MapAbstractCardToVirtue(SQLObject):
     abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
     virtueIndex = DatabaseIndex(virtue, unique=False)
 
+
+class MapAbstractCardToArtist(SQLObject):
+    class sqlmeta:
+        table = 'abs_artist_map'
+
+    tableversion = 1
+
+    abstractCard = ForeignKey('AbstractCard', notNull=True)
+    artist = ForeignKey('Artist', notNull=True)
+
+    abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
+    artistIndex = DatabaseIndex(artist, unique=False)
+
+
+class MapAbstractCardToKeyword(SQLObject):
+    class sqlmeta:
+        table = 'abs_keyword_map'
+
+    tableversion = 1
+
+    abstractCard = ForeignKey('AbstractCard', notNull=True)
+    keyword = ForeignKey('Keyword', notNull=True)
+
+    abstractCardIndex = DatabaseIndex(abstractCard, unique=False)
+    keywordIndex = DatabaseIndex(keyword, unique=False)
+
 # pylint: enable-msg=W0232, R0902, W0201, C0103
 
 # List of Tables to be created, dropped, etc.
 
-aObjectList = [ AbstractCard, Expansion,
+TABLE_LIST = [AbstractCard, Expansion,
                PhysicalCard, PhysicalCardSet,
                Rarity, RarityPair, Discipline, DisciplinePair,
                Clan, CardType, Sect, Title, Ruling, Virtue, Creed,
+               Artist, Keyword,
                # Mapping tables from here on out
                MapPhysicalCardToPhysicalCardSet,
                MapAbstractCardToRarityPair,
@@ -368,15 +495,21 @@ aObjectList = [ AbstractCard, Expansion,
                MapAbstractCardToTitle,
                MapAbstractCardToVirtue,
                MapAbstractCardToCreed,
+               MapAbstractCardToArtist,
+               MapAbstractCardToKeyword,
                ]
 # For reloading the Physical Card Sets
-aPhysicalSetList = [PhysicalCardSet,
+
+PHYSICAL_SET_LIST = [PhysicalCardSet,
         MapPhysicalCardToPhysicalCardSet]
 # For database upgrades, etc.
-aPhysicalList = [PhysicalCard] + aPhysicalSetList
+PHYSICAL_LIST = [PhysicalCard] + PHYSICAL_SET_LIST
+
+# Generically useful constant
+CRYPT_TYPES = ('Vampire', 'Imbued')
+
 
 # Object Maker API
-
 class SutekhObjectMaker(object):
     """Creates all kinds of SutekhObjects from simple strings.
 
@@ -392,7 +525,7 @@ class SutekhObjectMaker(object):
             return cInterface(sObj)
         except SQLObjectNotFound:
             sObj = cAbbreviation.canonical(sObj)
-            dKw = { 'name': sObj }
+            dKw = {'name': sObj}
             if bShortname:
                 dKw['shortname'] = cAbbreviation.shortname(sObj)
             if bFullname:
@@ -468,16 +601,27 @@ class SutekhObjectMaker(object):
         except SQLObjectNotFound:
             return Ruling(text=sText, code=sCode)
 
-# Abbreviation lookup based adapters
+    def make_keyword(self, sKeyword):
+        try:
+            return IKeyword(sKeyword)
+        except SQLObjectNotFound:
+            return Keyword(keyword=sKeyword)
 
+    def make_artist(self, sArtist):
+        try:
+            return IArtist(sArtist)
+        except SQLObjectNotFound:
+            return Artist(canonicalName=sArtist.lower(), name=sArtist)
+
+
+# Abbreviation lookup based adapters
 class StrAdaptMeta(type):
     """Metaclass for the string adaptors."""
-    # pylint: disable-msg=W0231, W0613, C0203
+    # pylint: disable-msg=W0231, C0203
     # W0231 - no point in calling type's init
-    # dDict, aBases, sName required by metaclass call signature
     # C0203 - pylint's buggy here, see
     # http://lists.logilab.org/pipermail/python-projects/2007-July/001249.html
-    def __init__(cls, sName, aBases, dDict):
+    def __init__(cls, _sName, _aBases, _dDict):
         cls.make_object_cache()
 
     # pylint: disable-msg=W0201
@@ -493,6 +637,7 @@ class StrAdaptMeta(type):
 
         return oObj
 
+
 class CardTypeAdapter(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -502,6 +647,7 @@ class CardTypeAdapter(object):
     def __new__(cls, sName):
         return cls.fetch(CardTypes.canonical(sName), CardType)
 
+
 class ClanAdapter(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -511,6 +657,7 @@ class ClanAdapter(object):
     def __new__(cls, sName):
         return cls.fetch(Clans.canonical(sName), Clan)
 
+
 class CreedAdapter(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -520,6 +667,7 @@ class CreedAdapter(object):
     def __new__(cls, sName):
         return cls.fetch(Creeds.canonical(sName), Creed)
 
+
 class DisciplineAdapter(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -529,6 +677,7 @@ class DisciplineAdapter(object):
     def __new__(cls, sName):
         return cls.fetch(Disciplines.canonical(sName), Discipline)
 
+
 class ExpansionAdapter(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -538,6 +687,7 @@ class ExpansionAdapter(object):
     def __new__(cls, sName):
         return cls.fetch(Expansions.canonical(sName), Expansion)
 
+
 class RarityAdapter(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -547,6 +697,7 @@ class RarityAdapter(object):
     def __new__(cls, sName):
         return cls.fetch(Rarities.canonical(sName), Rarity)
 
+
 class SectAdaptor(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -556,6 +707,7 @@ class SectAdaptor(object):
     def __new__(cls, sName):
         return cls.fetch(Sects.canonical(sName), Sect)
 
+
 class TitleAdapter(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -565,6 +717,7 @@ class TitleAdapter(object):
     def __new__(cls, sName):
         return cls.fetch(Titles.canonical(sName), Title)
 
+
 class VirtueAdapter(object):
     # pylint: disable-msg=E1101
     # metaclass confuses pylint
@@ -576,6 +729,7 @@ class VirtueAdapter(object):
 
 # Other Adapters
 
+
 class RarityPairAdapter(object):
     advise(instancesProvide=[IRarityPair], asAdapterForTypes=[tuple])
 
@@ -599,6 +753,7 @@ class RarityPairAdapter(object):
 
         return oPair
 
+
 class DisciplinePairAdapter(object):
     advise(instancesProvide=[IDisciplinePair], asAdapterForTypes=[tuple])
 
@@ -622,24 +777,54 @@ class DisciplinePairAdapter(object):
 
         return oPair
 
+
 class AbstractCardAdapter(object):
     advise(instancesProvide=[IAbstractCard], asAdapterForTypes=[basestring])
 
     def __new__(cls, sName):
         # pylint: disable-msg=E1101
         # SQLObject confuses pylint
-        return AbstractCard.byCanonicalName(sName.encode('utf8').lower())
+        try:
+            oCard = AbstractCard.byCanonicalName(sName.encode('utf8').lower())
+        except SQLObjectNotFound:
+            # Correct for common variations
+            sNewName = csv_to_canonical(sName)
+            if sNewName != sName:
+                oCard = AbstractCard.byCanonicalName(
+                        sNewName.encode('utf8').lower())
+            else:
+                raise
+        return oCard
+
 
 class RulingAdapter(object):
     advise(instancesProvide=[IRuling], asAdapterForTypes=[tuple])
 
     def __new__(cls, tData):
-        # pylint: disable-msg=E1101, W0612
+        # pylint: disable-msg=E1101,
         # SQLObject confuses pylint
-        # sCode is unused
-        sText, sCode = tData
+        sText, _sCode = tData
         return Ruling.byText(sText.encode('utf8'))
 
+
+class KeywordAdapter(object):
+    advise(instancesProvide=[IKeyword], asAdapterForTypes=[basestring])
+
+    def __new__(cls, sKeyword):
+        # pylint: disable-msg=E1101,
+        # SQLObject confuses pylint
+        return Keyword.byKeyword(sKeyword.encode('utf8'))
+
+
+class ArtistAdapter(object):
+    advise(instancesProvide=[IArtist], asAdapterForTypes=[basestring])
+
+    def __new__(cls, sArtistName):
+        # pylint: disable-msg=E1101,
+        # SQLObject confuses pylint
+        return Artist.byCanonicalName(sArtistName.encode('utf8').lower())
+
+
 class PhysicalCardSetAdapter(object):
     advise(instancesProvide=[IPhysicalCardSet], asAdapterForTypes=[basestring])
 
@@ -648,76 +833,179 @@ class PhysicalCardSetAdapter(object):
         # SQLObject confuses pylint
         return PhysicalCardSet.byName(sName.encode('utf8'))
 
+
 class PhysicalCardToAbstractCardAdapter(object):
     advise(instancesProvide=[IAbstractCard], asAdapterForTypes=[PhysicalCard])
 
+    __dCache = {}
+
+    @classmethod
+    def make_object_cache(cls):
+        cls.__dCache = {}
+
     def __new__(cls, oPhysCard):
-        return oPhysCard.abstractCard
+        oCard = cls.__dCache.get(oPhysCard.abstractCardID, None)
+        if oCard is None:
+            oCard = oPhysCard.abstractCard
+            cls.__dCache[oPhysCard.abstractCardID] = oCard
+        return oCard
+
 
 class PhysicalCardMappingToPhysicalCardAdapter(object):
-    advise(instancesProvide=[IPhysicalCard], asAdapterForTypes=
-            [MapPhysicalCardToPhysicalCardSet])
+    advise(instancesProvide=[IPhysicalCard],
+            asAdapterForTypes=[MapPhysicalCardToPhysicalCardSet])
+
+    __dCache = {}
+
+    @classmethod
+    def make_object_cache(cls):
+        cls.__dCache = {}
 
     def __new__(cls, oMapPhysCard):
-        return oMapPhysCard.physicalCard
+        oCard = cls.__dCache.get(oMapPhysCard.physicalCardID, None)
+        if oCard is None:
+            oCard = oMapPhysCard.physicalCard
+            cls.__dCache[oMapPhysCard.physicalCardID] = oCard
+        return oCard
+
 
 class PhysicalCardMappingToCardSetAdapter(object):
-    advise(instancesProvide=[IPhysicalCardSet], asAdapterForTypes=
-            [MapPhysicalCardToPhysicalCardSet])
+    advise(instancesProvide=[IPhysicalCardSet],
+            asAdapterForTypes=[MapPhysicalCardToPhysicalCardSet])
 
     def __new__(cls, oMapPhysCard):
         return oMapPhysCard.physicalCardSet
 
+
+class ExpansionNameAdapter(object):
+    """Converts PhysicalCard expansionID to name, used a lot in the gui"""
+    advise(instancesProvide=[IExpansionName], asAdapterForTypes=[PhysicalCard])
+
+    __dCache = {}
+    sUnknownExpansion = '  Unspecified Expansion'  # canonical version
+
+    @classmethod
+    def make_object_cache(cls):
+        cls.__dCache = {}
+
+    def __new__(cls, oPhysCard):
+        sExpName = cls.__dCache.get(oPhysCard.expansionID, None)
+        if sExpName is None:
+            if oPhysCard.expansionID:
+                sExpName = oPhysCard.expansion.name
+            else:
+                sExpName = cls.sUnknownExpansion
+            cls.__dCache[oPhysCard.expansionID] = sExpName
+        return sExpName
+
+
 class PhysicalCardMappingToAbstractCardAdapter(object):
-    advise(instancesProvide=[IAbstractCard], asAdapterForTypes=
-            [MapPhysicalCardToPhysicalCardSet])
+    advise(instancesProvide=[IAbstractCard],
+            asAdapterForTypes=[MapPhysicalCardToPhysicalCardSet])
+
+    __dCache = {}
+
+    @classmethod
+    def make_object_cache(cls):
+        cls.__dCache = {}
 
     def __new__(cls, oMapPhysCard):
-        return IAbstractCard(oMapPhysCard.physicalCard)
+        oCard = cls.__dCache.get(oMapPhysCard.physicalCardID, None)
+        if oCard is None:
+            oCard = IAbstractCard(oMapPhysCard.physicalCard)
+            cls.__dCache[oMapPhysCard.physicalCardID] = oCard
+        return oCard
 
 
 class PhysicalCardAdapter(object):
     advise(instancesProvide=[IPhysicalCard], asAdapterForTypes=[tuple])
 
+    __dCache = {}
+
+    @classmethod
+    def make_object_cache(cls):
+        cls.__dCache = {}
+        # pre-populate cache with mappings to commonly used
+        # physical card with None expansion.
+        # pylint: disable-msg=E1101
+        # SQLObject confuses pylint
+        try:
+            for oPhysicalCard in PhysicalCard.select(
+                    PhysicalCard.q.expansion == None):
+                oAbsCard = oPhysicalCard.abstractCard
+                cls.__dCache[(oAbsCard.id, None)] = oPhysicalCard
+        except AttributeError:
+            # Old SQLObject doesn't like this construction if the database
+            # is empty, so, as we can't sensibly fill the cache anyway, we
+            # just skip
+            pass
+
     def __new__(cls, tData):
         # pylint: disable-msg=E1101
         # SQLObject confuses pylint
         oAbsCard, oExp = tData
-        oPhysicalCard = PhysicalCard.selectBy(abstractCard=oAbsCard,
-                                              expansion=oExp).getOne()
+        # oExp may be None, so we don't use oExp.id here
+        oPhysicalCard = cls.__dCache.get((oAbsCard.id, oExp), None)
+        if oPhysicalCard is None:
+            oPhysicalCard = PhysicalCard.selectBy(abstractCard=oAbsCard,
+                    expansion=oExp).getOne()
+            cls.__dCache[(oAbsCard.id, oExp)] = oPhysicalCard
         return oPhysicalCard
 
+# pylint: enable-msg=C0111
+
+
 # Flushing
 
+def make_adaptor_caches():
+    """Flush all adaptor caches."""
+    for cAdaptor in [ExpansionAdapter, RarityAdapter, DisciplineAdapter,
+                      ClanAdapter, CardTypeAdapter, SectAdaptor, TitleAdapter,
+                      VirtueAdapter, CreedAdapter, DisciplinePairAdapter,
+                      RarityPairAdapter, PhysicalCardAdapter,
+                      PhysicalCardMappingToPhysicalCardAdapter,
+                      PhysicalCardToAbstractCardAdapter,
+                      PhysicalCardMappingToAbstractCardAdapter,
+                      ExpansionNameAdapter,
+                      ]:
+        cAdaptor.make_object_cache()
+
+
 def flush_cache():
     """Flush all the object caches - needed before importing new card lists
        and such"""
-    for cAdaptor in [ ExpansionAdapter, RarityAdapter, DisciplineAdapter,
-                      ClanAdapter, CardTypeAdapter, SectAdaptor, TitleAdapter,
-                      VirtueAdapter, CreedAdapter, DisciplinePairAdapter,
-                      RarityPairAdapter ]:
-        cAdaptor.make_object_cache()
-
     for oJoin in AbstractCard.sqlmeta.joins:
         if type(oJoin) is SOCachedRelatedJoin:
             oJoin.flush_cache()
 
+    make_adaptor_caches()
+
+
 def init_cache():
     """Initiliase the cached join tables."""
     for oJoin in AbstractCard.sqlmeta.joins:
         if type(oJoin) is SOCachedRelatedJoin:
             oJoin.init_cache()
 
-# backport these so we can use the SL stuff.
-# The other fixes around these functions aren't backported yet
+    make_adaptor_caches()
+
+
+# helper conversion functions
+# We define them here to avoid circular imports, since pretty much everything
+# else requires SutekhObjects
+
+
 def canonical_to_csv(sName):
     """Moves articles to the end of the name"""
     if sName.startswith('The '):
         sName = sName[4:] + ", The"
     elif sName.startswith('An '):
         sName = sName[3:] + ", An"
+    elif sName.startswith('A '):
+        sName = sName[2:] + ", A"
     return sName
 
+
 def csv_to_canonical(sName):
     """Moves articles from the end back to the start - reverses
        cannonical_to_csv"""
@@ -726,7 +1014,8 @@ def csv_to_canonical(sName):
         sName = "The " + sName[:-5]
     elif sName.lower().endswith(', an'):
         sName = "An " + sName[:-4]
+    elif sName.lower().endswith(', a'):
+        sName = "A " + sName[:-3]
     # The result might be mixed case, but, as we will feed this into
     # IAbstractCard in most cases, that won't matter
     return sName
-
