@@ -8,35 +8,23 @@
 """gtk.TreeView classes for displaying the card list."""
 
 import gtk
-import unicodedata
+from sutekh.gui.FilteredView import FilteredView
 from sutekh.gui.FilterDialog import FilterDialog
-from sutekh.gui.SearchDialog import SearchDialog
 
-class CardListViewListener(object):
-    """Listens to changes, i.e. .set_card_text(...) to CardListViews."""
-    def set_card_text(self, sCardName, sExpansion):
-        """The CardListViw has called set_card_text on the CardText pane"""
-        pass
 
-class CardListView(gtk.TreeView, object):
+class CardListView(FilteredView):
     """Base class for all the card list views in Sutekh."""
-    # pylint: disable-msg=R0904, R0902
-    # gtk.Widget, so many public methods. We need to keep state, so many attrs
+    # pylint: disable-msg=R0904, R0902, R0901
+    # R0904 - gtk.Widget, so many public methods
+    # R0902 - We need to track a fair amount of state, so many attributes
+    # R0901 - many ancestors, due to our object hierachy on top of the quite
+    # deep gtk one
+
     def __init__(self, oController, oMainWindow, oModel, oConfig):
-        # Although MainWindow usually contains a config_file property,
-        # when we come in from the GuiCardLookup, we just have oConfig
-        self._oModel = oModel
-        self._oController = oController
-        self._oMainWin = oMainWindow
-        self._oConfig = oConfig
-        self.dListeners = {} # dictionary of CardListViewListeners
+        super(CardListView, self).__init__(oController, oMainWindow,
+                oModel, oConfig)
 
-        super(CardListView, self).__init__(self._oModel)
-
-        # Selecting rows
-        self._oSelection = self.get_selection()
-        self._oSelection.set_mode(gtk.SELECTION_MULTIPLE)
-        self._aOldSelection = []
+        self.set_select_multiple()
 
         self._oSelection.connect('changed', self.card_selected)
 
@@ -58,17 +46,14 @@ class CardListView(gtk.TreeView, object):
 
         # Activating rows
         self.connect('row-activated', self.card_activated)
+
+        # special handling for select all
+        self.connect('select-all', self.select_all)
         # Key combination for searching
 
-        # Text searching of card names
-        self.set_search_equal_func(self.compare, None)
-        # Search dialog
-        # Entry item for text searching
-        self._oSearchDialog = SearchDialog(self, self._oMainWin)
-
         # Drag and Drop
-        aTargets = [ ('STRING', 0, 0),      # second 0 means TARGET_STRING
-                     ('text/plain', 0, 0) ] # and here
+        aTargets = [('STRING', 0, 0),      # second 0 means TARGET_STRING
+                    ('text/plain', 0, 0)]  # and here
 
         self.drag_source_set(gtk.gdk.BUTTON1_MASK | gtk.gdk.BUTTON3_MASK,
                              aTargets,
@@ -81,79 +66,8 @@ class CardListView(gtk.TreeView, object):
         self.connect('drag_data_get', self.drag_card)
         self.connect('drag_data_delete', self.drag_delete)
         self.connect('drag_data_received', self.card_drop)
-        self.bReentrant = False
         self.bSelectTop = 0
 
-        # Filtering Dialog
-        self._oFilterDialog = None
-        self.set_name('normal_view')
-
-        # Grid Lines
-        if hasattr(self, 'set_grid_lines'):
-            self.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_BOTH)
-
-
-    # Listener helper functions
-
-    def add_listener(self, oListener):
-        """Add a listener to the list."""
-        self.dListeners[oListener] = None
-
-    def remove_listener(self, oListener):
-        """Remove a listener from the list."""
-        del self.dListeners[oListener]
-
-    def load(self):
-        """Called when the model needs to be reloaded."""
-        if hasattr(self._oMainWin, 'set_busy_cursor'):
-            self._oMainWin.set_busy_cursor()
-        self.freeze_child_notify()
-        self.set_model(None)
-        self._oModel.load()
-        self.set_model(self._oModel)
-        self.thaw_child_notify()
-        if hasattr(self._oMainWin, 'restore_cursor'):
-            self._oMainWin.restore_cursor()
-
-    def __set_row_status(self, dExpandedDict):
-        """Attempt to expand the rows listed in dExpandedDict."""
-        for oPath in dExpandedDict:
-            # pylint: disable-msg=W0704
-            # Paths may disappear, so this error can be ignored
-            try:
-                sCardName = self._oModel.get_card_name_from_path(oPath)
-                if sCardName == dExpandedDict[oPath]:
-                    self.expand_to_path(oPath)
-            except ValueError:
-                pass
-        return False
-
-    def reload_keep_expanded(self):
-        """Reload with current expanded state.
-
-           Attempt to reload the card list, keeping the existing structure
-           of expanded rows.
-           """
-        # Internal helper functions
-        # See what's expanded
-        dExpandedDict = {}
-        self._oModel.foreach(self.__get_row_status, dExpandedDict)
-        # Reload, but use cached info
-        self.load()
-        # Re-expand stuff
-        self.__set_row_status(dExpandedDict)
-
-    # Introspection
-
-    # pylint: disable-msg=W0212
-    # We allow access via these properties (for plugins)
-    mainwindow = property(fget=lambda self: self._oMainWin,
-            doc="The parent window used for dialogs, etc.")
-    searchdialog = property(fget=lambda self: self._oSearchDialog,
-            doc="The search dialog.")
-    # pylint: enable-msg=W0212
-
-
     def can_select(self, oPath):
         """disable selecting top level rows"""
         if self.bSelectTop > 0:
@@ -171,101 +85,14 @@ class CardListView(gtk.TreeView, object):
            a single row that is in the selection, we DON'T change
            the selection, but we do update the card text and so on.
            """
-        if self.bReentrant:
-            # This is here because we alter the selection inside
-            # this function (resulting in a nested call).
-            # self.bReentrant is set and unset below.
+        # Check with helper function first
+        oPath = self.row_selected(oSelection)
+        if not oPath:
             return False
 
-        # If the selection is empty, clear everything and return
-        if oSelection.count_selected_rows() <= 0:
-            self._aOldSelection = []
-            return False
-
-        # pylint: disable-msg=W0612
-        # We're only interested in aList
-        oModel, aList = oSelection.get_selected_rows()
-        # Implement the non default selection behaviour.
-        tCursorPos = self.get_cursor()
-        if len(aList) == 1 and len(self._aOldSelection) > 1 and \
-            tCursorPos[0] == aList[0] and aList[0] in self._aOldSelection:
-            # reset the list to it's previous state, but set
-            # displayed card to this one
-            try:
-                self.bReentrant = True
-                for oPath in self._aOldSelection:
-                    oSelection.select_path(oPath)
-            finally:
-                self.bReentrant = False
-            oPath = aList[0]
-        else:
-            oModel, aList = oSelection.get_selected_rows()
-            if not aList:
-                self._aOldSelection = []
-                return False
-
-            if len(aList) <= len(self._aOldSelection):
-                oPath = aList[-1]
-            else:
-                # Find the last entry in aList that's not in _aOldSelection
-                oPath = [x for x in aList if x not in self._aOldSelection][-1]
-            self._aOldSelection = aList
-
-        sCardName = self._oModel.get_card_name_from_path(oPath)
-        self._oController.set_card_text(sCardName)
-        sExpansion = self._oModel.get_exp_name_from_path(oPath)
-        if not sExpansion:
-            sExpansion = ''
-        for oListener in self.dListeners:
-            oListener.set_card_text(sCardName, sExpansion)
-
-    # Filtering
-
-    def get_filter(self, oMenu, sDefaultFilter=None):
-        """Get the Filter from the FilterDialog.
-
-           oMenu is a menu item to toggle if it exists.
-           sDefaultFilterName is the name of the default filter
-           to set when the dialog is created. This is intended for
-           use by GuiCardLookup."""
-        if self._oFilterDialog is None:
-            self._oFilterDialog = FilterDialog(self._oMainWin,
-                    self._oConfig, self._oController.filtertype,
-                    sDefaultFilter)
-
-        self._oFilterDialog.run()
-
-        if self._oFilterDialog.was_cancelled():
-            return # Change nothing
-
-        oFilter = self._oFilterDialog.get_filter()
-        if oFilter != None:
-            self._oModel.selectfilter = oFilter
-            if not self._oModel.applyfilter:
-                # If a filter is set, automatically apply
-                if oMenu:
-                    oMenu.set_apply_filter(True)
-                else:
-                    self._oModel.applyfilter = True
-            else:
-                # Filter Changed, so reload
-                self.load()
-        else:
-            # Filter is set to blank, so we treat this as disabling
-            # Filter
-            if self._oModel.applyfilter:
-                if oMenu:
-                    oMenu.set_apply_filter(False)
-                else:
-                    self._oModel.applyfilter = True
-            else:
-                self.load()
-
-    def run_filter(self, bState):
-        """Enable or diable the current filter based on bState"""
-        if self._oModel.applyfilter != bState:
-            self._oModel.applyfilter = bState
-            self.load()
+        # Change card text view as required
+        oPhysCard = self._oModel.get_physical_card_from_path(oPath)
+        self._oController.set_card_text(oPhysCard)
 
     def process_selection(self):
         """Create a dictionary from the selection.
@@ -273,9 +100,9 @@ class CardListView(gtk.TreeView, object):
            Entries are of the form sCardName : {sExpansion1 : iCount1, ... }
            for use in drag-'n drop and elsewhere.
            """
-        oModel, oPathList = self._oSelection.get_selected_rows()
+        oModel, aPathList = self._oSelection.get_selected_rows()
         dSelectedData = {}
-        for oPath in oPathList:
+        for oPath in aPathList:
             sCardName, sExpansion, iCount, iDepth = \
                     oModel.get_all_from_path(oPath)
             if iDepth == 0:
@@ -308,8 +135,11 @@ class CardListView(gtk.TreeView, object):
         return dSelectedData
 
     def get_selection_as_string(self):
-        """Get a string representing the current selection."""
-        if self._oSelection.count_selected_rows()<1:
+        """Get a string representing the current selection.
+
+           Because of how pygtk handles drag-n-drop data, we need to
+           create a string representating the card data."""
+        if self._oSelection.count_selected_rows() < 1:
             return ''
         dSelectedData = self.process_selection()
         # Create selection data structure
@@ -318,25 +148,17 @@ class CardListView(gtk.TreeView, object):
         for sCardName in dSelectedData:
             for sExpansion, iCount in dSelectedData[sCardName].iteritems():
                 sSelectData += '\n%(count)d\n%(name)s\n%(expansion)s' % {
-                        'count' : iCount,
-                        'name' : sCardName,
-                        'expansion' : sExpansion,
+                        'count': iCount,
+                        'name': sCardName,
+                        'expansion': sExpansion,
                         }
         return sSelectData
 
     # Drag and Drop
     # Sub-classes should override as needed.
-    # pylint: disable-msg=R0201
-    # These need to be available to children as methods
 
     def split_selection_data(self, sSelectionData):
         """Helper function to subdivide selection string into bits again"""
-        if sSelectionData == '':
-            return 'None', ['']
-        aLines = sSelectionData.splitlines()
-        sSource = aLines[0]
-        if sSource == "Sutekh Pane:" or sSource == 'Card Set:':
-            return sSource, aLines
         # Construct list of (iCount, sCardName, sExpansion) tuples
         def true_expansion(sExpand):
             """Convert back from the 'None' placeholder in the string"""
@@ -347,11 +169,17 @@ class CardListView(gtk.TreeView, object):
                 return None
             else:
                 return sExpand
+
+        sSource, aLines = \
+                super(CardListView, self).split_selection_data(sSelectionData)
+        if sSource in ("None", "Sutekh Pane:", "Card Set:"):
+            # Not cards that were dragged, so just return
+            return sSource, aLines
         aCardInfo = zip([int(x) for x in aLines[1::3]], aLines[2::3],
                 [true_expansion(x) for x in aLines[3::3]])
         return sSource, aCardInfo
 
-    # pylint: disable-msg=R0913, W0613
+    # pylint: disable-msg=R0913
     # arguments as required by the function signature
 
     def drag_card(self, oBtn, oContext, oSelectionData, oInfo, oTime):
@@ -364,7 +192,7 @@ class CardListView(gtk.TreeView, object):
             return
         oSelectionData.set(oSelectionData.target, 8, sSelectData)
 
-    def drag_delete(self, oBtn, oContext, oData):
+    def drag_delete(self, oBtn, oContext):
         """Default drag-delete handler"""
         pass
 
@@ -381,12 +209,7 @@ class CardListView(gtk.TreeView, object):
 
     # Card name searching
 
-    @staticmethod
-    def to_ascii(sName):
-        """Convert a card name or key to a canonical ASCII form."""
-        return unicodedata.normalize('NFKD', sName).encode('ascii','ignore')
-
-    def compare(self, oModel, iColumn, sKey, oIter, oData):
+    def compare(self, oModel, _iColumn, sKey, oIter, _oData):
         """Compare the entered text to the card names."""
         if oModel.iter_depth(oIter) == 2:
             # Don't succeed for expansion items
@@ -413,7 +236,7 @@ class CardListView(gtk.TreeView, object):
                         # Bail out, as compare will find the match for us
                         return True
                     oChildIter = self._oModel.iter_next(oChildIter)
-                return True # No matches, so bail
+                return True  # No matches, so bail
 
         sCardName = self._oModel.get_name_from_iter(oIter)[:iLenKey].lower()
         if self.to_ascii(sCardName).startswith(sKey) or \
@@ -422,34 +245,23 @@ class CardListView(gtk.TreeView, object):
 
         return True
 
-    # Helper function used by reload_keep_expanded
-    # Various arguments required by function signatures
-    def __get_row_status(self, oModel, oPath, oIter, dExpandedDict):
-        """Create a dictionary of rows and their expanded status."""
-        if self.row_expanded(oPath):
-            dExpandedDict.setdefault(oPath,
-                    self._oModel.get_card_name_from_path(oPath))
-        return False # Need to process the whole list
+    # pylint: enable-msg=R0913
 
     # Activating Rows
-    def card_activated(self, oTree, oPath, oColumn):
+    def card_activated(self, _oTree, oPath, _oColumn):
         """Update card text and notify listeners when a card is selected."""
-        sCardName = self._oModel.get_card_name_from_path(oPath)
-        self._oController.set_card_text(sCardName)
-        for oListener in self.dListeners:
-            oListener.set_card_text(sCardName, '')
+        oPhysCard = self._oModel.get_physical_card_from_path(oPath)
+        self._oController.set_card_text(oPhysCard)
 
     # Selecting
 
-    def force_cursor_move(self, oTreeView, iStep, iCount):
+    def force_cursor_move(self, _oTreeView, _iStep, _iCount):
         """Special handling for move events for buggy gtk events.
 
            We need to allow the selection of top level items when
            moving the cursor over them
            """
-        # pylint: disable-msg=W0612
-        # We're only interested in oCurPath
-        (oCurPath, oColumn) = self.get_cursor()
+        oCurPath, _oColumn = self.get_cursor()
         if self._oModel.iter_parent(self._oModel.get_iter(oCurPath)) is None:
             # Root node, so need to force the move
             self.bSelectTop = 2
@@ -461,4 +273,29 @@ class CardListView(gtk.TreeView, object):
         # anything else funky
         return False
 
-    # pylint: enable-msg=R0913, W0613
+    # Filtering
+
+    def _get_filter_dialog(self, sDefaultFilter):
+        """Create the filter dialog for this view."""
+        self._oFilterDialog = FilterDialog(self._oMainWin, self._oConfig,
+                self._oController.filtertype, sDefaultFilter)
+        return True
+
+    def make_drag_icon(self, _oWidget, oDragContext):
+        """Custom drag icon for dragging cards"""
+        # We use STOCK_DND_MULTIPLE for multiple cards, otherwise
+        # STOCK_DND for single row selected
+        iNumSelected = self._oSelection.count_selected_rows()
+        if iNumSelected > 1:
+            self.drag_source_set_icon_stock(gtk.STOCK_DND_MULTIPLE)
+        elif iNumSelected == 1:
+            self.drag_source_set_icon_stock(gtk.STOCK_DND)
+        else:
+            # Nothing selected, so we're dragging the entire pane, so use
+            # the pane icon
+            self.frame.make_drag_icon(self, oDragContext)
+
+    def select_all(self, _oWidget):
+        """Expand the tree and select all the nodes"""
+        self.expand_all()
+        self._oSelection.select_all()
