@@ -13,23 +13,24 @@ import pango
 import gobject
 from sqlobject import SQLObjectNotFound
 from sutekh.core.SutekhObjects import AbstractCard, PhysicalCard, IExpansion, \
-        Expansion, IPhysicalCard
+        Expansion, IPhysicalCard, IAbstractCard
 from sutekh.core.CardLookup import AbstractCardLookup, PhysicalCardLookup, \
         ExpansionLookup, LookupFailed, best_guess_filter
 from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error
 from sutekh.gui.CellRendererSutekhButton import CellRendererSutekhButton
 from sutekh.gui.PhysicalCardView import PhysicalCardView
 from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
+from sutekh.gui.FilterModelPanes import add_accel_to_button
 
 
-def add_accel_to_button(oButton, sAccelKey, oAccelGroup, sToolTip=None):
-    """Creates a button using an gtk.AccelLabel to display the accelerator"""
-    (iKeyVal, iMod) = gtk.accelerator_parse(sAccelKey)
-    if iKeyVal != 0:
-        oButton.add_accelerator('clicked', oAccelGroup, iKeyVal, iMod,
-                gtk.ACCEL_VISIBLE)
-    if sToolTip and hasattr(oButton,'set_tooltip_markup'):
-        oButton.set_tooltip_markup(sToolTip)
+NO_CARD = "  No Card"
+
+
+def _sort_replacement(oModel, oIter1, oIter2):
+    """Sort replacement, honouring spaces"""
+    oVal1 = oModel.get_value(oIter1, 2)
+    oVal2 = oModel.get_value(oIter2, 2)
+    return cmp(oVal1, oVal2)
 
 
 class DummyController(object):
@@ -39,10 +40,11 @@ class DummyController(object):
 
     filtertype = property(fget=lambda self: self.sFilterType)
 
-    def set_card_text(self, sCardName):
+    def set_card_text(self, _oCard):
         """Ignore card text updates."""
         pass
 
+
 class ACLLookupView(PhysicalCardView):
     """Specialised version for the Card Lookup."""
     # pylint: disable-msg=R0904
@@ -57,15 +59,18 @@ class ACLLookupView(PhysicalCardView):
 
     def get_selected_card(self):
         """Return the selected card with a dummy expansion of ''."""
-        sNewName = 'No Card'
+        sNewName = NO_CARD
         oModel, aSelection = self.get_selection().get_selected_rows()
         for oPath in aSelection:
-            # pylint: disable-msg=W0612
-            # only interested in sNewName and sExpansion
-            sNewName, sExpansion, iCount, iDepth = \
+            sNewName, _sExpansion, _iCount, _iDepth = \
                 oModel.get_all_from_path(oPath)
         return sNewName, ''
 
+    def cleanup(self):
+        """Call model cleanup to remove listeners, etc."""
+        self._oModel.cleanup()
+
+
 class PCLLookupView(PhysicalCardView):
     """Also show current allocation of cards in the physical card view."""
     # pylint: disable-msg=R0904
@@ -79,18 +84,21 @@ class PCLLookupView(PhysicalCardView):
 
     def get_selected_card(self):
         """Return the selected card and expansion."""
-        sNewName = 'No Card'
+        sNewName = NO_CARD
         sExpansion = '  Unpecified Expansion'
         oModel, aSelection = self.get_selection().get_selected_rows()
         for oPath in aSelection:
-            # pylint: disable-msg=W0612
-            # only interested in sNewName and sExpansion
-            sNewName, sExpansion, iCount, iDepth = \
+            sNewName, sExpansion, _iCount, _iDepth = \
                 oModel.get_all_from_path(oPath)
         if sExpansion is None:
             sExpansion = ''
         return sNewName, sExpansion
 
+    def cleanup(self):
+        """Call model cleanup to remove listeners, etc."""
+        self._oModel.cleanup()
+
+
 class ReplacementTreeView(gtk.TreeView):
     """A TreeView which tracks the current set of replacement cards."""
     # pylint: disable-msg=R0904
@@ -105,27 +113,36 @@ class ReplacementTreeView(gtk.TreeView):
            appended.
            """
         # ListStore: count, missing card, replacement
+
+        self._dToolTips = {}
+
         oModel = gtk.ListStore(gobject.TYPE_INT, gobject.TYPE_STRING,
-                               gobject.TYPE_STRING)
+                               gobject.TYPE_STRING, gobject.TYPE_INT)
 
         super(ReplacementTreeView, self).__init__(oModel)
         self.oCardListView = oCardListView
         self.oModel = oModel
         self.oFilterToggleButton = oFilterToggleButton
-        self.set_enable_search(False) # Not much point searching this tree
+        self.set_enable_search(False)  # Not much point searching this tree
 
         self._create_text_column('Missing Card', 1)
         self._create_text_column('Replace Card', 2)
 
         self._create_button_column(gtk.STOCK_OK, 'Set',
                 'Use the selected card',
-                self._set_to_selection) # use selected card
+                self._set_to_selection)  # use selected card
         self._create_button_column(gtk.STOCK_REMOVE, 'Ignore',
                 'Ignore the current card',
-                self._set_ignore) # ignore current card
+                self._set_ignore)  # ignore current card
         self._create_button_column(gtk.STOCK_FIND, 'Filter',
                 'Filter on best guess',
-                self._set_filter) # filter out best guesses
+                self._set_filter)  # filter out best guesses
+
+        self.set_property('has-tooltip', True)
+        self.connect('query-tooltip', self._show_tooltip)
+
+        oModel.set_sort_func(2, _sort_replacement)
+        oModel.set_sort_column_id(2, 0)
 
     # utility methods to simplify column creation
     def _create_button_column(self, oIcon, sLabel, sToolTip, fClicked):
@@ -134,40 +151,67 @@ class ReplacementTreeView(gtk.TreeView):
         oCell = CellRendererSutekhButton(bShowIcon=True)
         oCell.load_icon(oIcon, self)
         oLabel = gtk.Label(sLabel)
-        if hasattr(oLabel,'set_tooltip_text'):
-            # GTK < 2.12 doesn't have the new tooltip API
-            oLabel.set_tooltip_text(sToolTip)
         oColumn = gtk.TreeViewColumn("", oCell)
         oColumn.set_widget(oLabel)
         oColumn.set_fixed_width(22)
         oColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
         self.append_column(oColumn)
         oCell.connect('clicked', fClicked)
+        self._dToolTips[oColumn] = sToolTip
+
+    # pylint: disable-msg=R0913
+    # Need this many arguments from function signature
+    # Show tooltips for the column buttons
+    def _show_tooltip(self, _oWidget, _iXPos, _iYPos, _bKeyboard, oToolTip):
+        """Test if a tooltip should be shown"""
+        sToolTip = None
+        # The positions passed in aren't relative to the right window for
+        # get_path, so we query position directly
+        iXPos, iYPos, _oIgnore = self.get_bin_window().get_pointer()
+        tRes = self.get_path_at_pos(iXPos, iYPos)
+        if tRes:
+            # path returned may be none
+            sToolTip = self._dToolTips.get(tRes[1], None)
+        if sToolTip:
+            oToolTip.set_markup(sToolTip)
+            return True
+        return False
+
+    # pylint: enable-msg=R0913
 
     def _create_text_column(self, sLabel, iColumn):
         """Create a text column, using iColumn from the model"""
         oCell = gtk.CellRendererText()
         oCell.set_property('style', pango.STYLE_ITALIC)
-        oColumn = gtk.TreeViewColumn(sLabel, oCell, text=iColumn)
+        oColumn = gtk.TreeViewColumn(sLabel, oCell, text=iColumn, weight=3)
         oColumn.set_expand(True)
         oColumn.set_sort_column_id(iColumn)
         self.append_column(oColumn)
 
-    # pylint: disable-msg=W0613
-    # argument lists determined by callback signature
+    def _fix_selection(self, sName):
+        """Fix the cursor position to accomodate the result of the rows
+           being sorted"""
+        oIter = self.oModel.get_iter_first()
+        while oIter:
+            sCurName = self.oModel.get_value(oIter, 1)
+            if sName == sCurName:
+                self.set_cursor(self.oModel.get_path(oIter))
+                return
+            oIter = self.oModel.iter_next(oIter)
 
-    def _set_to_selection(self, oCell, oPath):
+    def _set_to_selection(self, _oCell, oPath):
         """Set the replacement card to the selected entry"""
 
         # We handle PhysicalCards on a like-for-like matching case.
         # For cases where the user selects an expansion with too few
-        # cards, but where there are enough phyiscal cards, we do the
+        # cards, but where there are enough physical cards, we do the
         # best we can
 
         oIter = self.oModel.get_iter(oPath)
+        sName = self.oModel.get_value(oIter, 1)
 
         sNewName, sExpansion = self.oCardListView.get_selected_card()
-        if sNewName == 'No Card':
+        if sNewName == NO_CARD:
             do_complaint_error("Please select a card")
             return
 
@@ -177,19 +221,22 @@ class ReplacementTreeView(gtk.TreeView):
             sReplaceWith = sNewName
 
         self.oModel.set_value(oIter, 2, sReplaceWith)
+        self.oModel.set_value(oIter, 3, pango.WEIGHT_NORMAL)
+        gobject.timeout_add(1, self._fix_selection, sName)
 
-    def _set_ignore(self, oCell, oPath):
+    def _set_ignore(self, _oCell, oPath):
         """Mark the card as not having a replacement."""
         oIter = self.oModel.get_iter(oPath)
-        self.oModel.set_value(oIter, 2, "No Card")
+        sName = self.oModel.get_value(oIter, 1)
+        self.oModel.set_value(oIter, 2, NO_CARD)
+        self.oModel.set_value(oIter, 3, pango.WEIGHT_BOLD)
+        gobject.timeout_add(1, self._fix_selection, sName)
 
-    def _set_filter(self, oCell, oPath):
+    def _set_filter(self, _oCell, oPath):
         """Set the card list filter to the best guess filter for this card."""
         oIter = self.oModel.get_iter(oPath)
         sFullName = self.oModel.get_value(oIter, 1)
-        # pylint: disable-msg=W0612
-        # we ignore sExp here
-        sName, sExp = self.parse_card_name(sFullName)
+        sName, _sExp = self.parse_card_name(sFullName)
 
         oFilter = best_guess_filter(sName)
         self.oCardListView.get_model().selectfilter = oFilter
@@ -199,23 +246,26 @@ class ReplacementTreeView(gtk.TreeView):
         else:
             self.oCardListView.load()
 
-    def run_filter_dialog(self, oButton):
+    def run_filter_dialog(self, _oButton):
         """Display the filter dialog and apply results."""
         self.oCardListView.get_filter(None, "CardName in $var0")
         self.oFilterToggleButton.set_active(
             self.oCardListView.get_model().applyfilter)
 
-    def toggle_apply_filter(self, oButton):
+    def toggle_apply_filter(self, _oButton):
         """Toggle whether the filter is applied."""
         self.oCardListView.get_model().applyfilter = \
             self.oFilterToggleButton.get_active()
         self.oCardListView.load()
 
-    def show_search(self, oButton):
-        """Popup the search dialog"""
-        self.oCardListView.searchdialog.show_all()
+    def toggle_show_illegal(self, oButton):
+        """Toggle whether the filter is applied."""
+        self.oCardListView.get_model().hideillegal = not oButton.get_active()
+        self.oCardListView.load()
 
-    # pylint: enable-msg=W0613
+    def show_search(self, _oButton):
+        """Popup the search dialog"""
+        self.oCardListView.emit('start-interactive-search')
 
     NAME_RE = re.compile(r"^(?P<name>.*?)( \[(?P<exp>[^]]+)\])?$")
 
@@ -226,6 +276,7 @@ class ReplacementTreeView(gtk.TreeView):
         assert oMatch is not None
         return oMatch.group('name'), oMatch.group('exp')
 
+
 class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
     """Lookup AbstractCards. Use the user as the AI if a simple lookup fails.
        """
@@ -239,24 +290,40 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
 
            Provides an implementation for AbstractCardLookup.
            """
+        # pylint: disable-msg=R0912
+        # Lots of different cases to consider, to several branches
         dCards = {}
         dUnknownCards = {}
 
+        aNewNames = []  # If we need to recode the name
+
         for sName in aNames:
             if not sName:
                 # None here is an explicit ignore from the lookup cache
                 dCards[sName] = None
             else:
+                # Check we can encode the name properly
                 try:
-                    # pylint: disable-msg=E1101
-                    # SQLObject methods confuse pylint
-                    oAbs = AbstractCard.byCanonicalName(
-                                sName.encode('utf8').lower())
-                    # pylint: enable-msg=E1101
+                    _sTemp = sName.encode('utf8')
+                except UnicodeDecodeError:
+                    # Wrong assumptions somewhere - let the user sort it
+                    # out
+                    # We bounce through unicode to ensure
+                    # we have something usable everywhere
+                    # FIXME: Fix best guess filter and card lookup
+                    # code so we don't need to encode back to ascii
+                    sName = sName.decode('ascii', 'replace').encode('ascii',
+                            'replace')
+
+                try:
+                    # Use IAbstractCard to cover more variations
+                    oAbs = IAbstractCard(sName.encode('utf8'))
                     dCards[sName] = oAbs
                 except SQLObjectNotFound:
                     dUnknownCards[sName] = None
 
+                aNewNames.append(sName)
+
         if dUnknownCards:
             if not self._handle_unknown_abstract_cards(dUnknownCards, sInfo):
                 raise LookupFailed("Lookup of missing cards aborted by the"
@@ -283,7 +350,7 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
             else:
                 return dUnknownCards[sName]
 
-        return [new_card(sName) for sName in aNames]
+        return [new_card(sName) for sName in aNewNames]
 
     def physical_lookup(self, dCardExpansions, dNameCards, dNameExps, sInfo):
         """Lookup missing physical cards.
@@ -302,7 +369,8 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
                 oExpansion = dNameExps[sExpansionName]
                 if iCnt > 0:
                     try:
-                        aCards.extend([IPhysicalCard((oAbs, oExpansion))]*iCnt)
+                        aCards.extend(
+                                [IPhysicalCard((oAbs, oExpansion))] * iCnt)
                     except SQLObjectNotFound:
                         dUnknownCards[(oAbs.name, sExpansionName)] = iCnt
 
@@ -379,11 +447,12 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
 
         # Populate the model with the card names and best guesses
         for (sName, sExpName), iCnt in dUnknownCards.items():
-            sBestGuess = "No Card"
+            sBestGuess = NO_CARD
             sFullName = "%s [%s]" % (sName, sExpName)
 
             oIter = oModel.append(None)
-            oModel.set(oIter, 0, iCnt, 1, sFullName, 2, sBestGuess)
+            oModel.set(oIter, 0, iCnt, 1, sFullName, 2, sBestGuess,
+                    3, pango.WEIGHT_BOLD)
 
         oUnknownDialog.vbox.show_all()
         oPhysCardView.load()
@@ -391,6 +460,8 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
         iResponse = oUnknownDialog.run()
         oUnknownDialog.destroy()
 
+        oPhysCardView.cleanup()
+
         if iResponse == gtk.RESPONSE_OK:
             # For cards marked as replaced, add them to the list of
             # Physical Cards
@@ -402,13 +473,13 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
                 sNewName, sNewExpName = \
                     oReplacementView.parse_card_name(sNewFullName)
 
-                if sNewName != "No Card":
+                if sNewName != NO_CARD:
                     iCnt = dUnknownCards[(sName, sExpName)]
 
                     # Find First physical card that matches this name
                     # that's not in aPhysCards
                     oPhys = self._lookup_new_phys_card(sNewName, sNewExpName)
-                    aPhysCards.extend([oPhys]*iCnt)
+                    aPhysCards.extend([oPhys] * iCnt)
 
                 oIter = oModel.iter_next(oIter)
             return True
@@ -444,7 +515,6 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
 
         return oUnknownDialog, oHBox
 
-
     def _fill_dialog(self, oHBox, oView):
         """Handle the view setup and default buttons for the lookup
            dialog."""
@@ -464,12 +534,16 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
 
         oFilterDialogButton = gtk.Button('Specify Filter')
         oFilterApplyButton = gtk.CheckButton("Apply Filter")
+        oLegalButton = gtk.CheckButton("Show illegal cards")
         oSearchButton = gtk.Button("Search List")
         add_accel_to_button(oFilterDialogButton, "<Ctrl>s", oAccelGroup,
                 'Open the Filter Editing Dialog.\nShortcut :<b>Ctrl-S</b>')
         add_accel_to_button(oFilterApplyButton, "<Ctrl>t", oAccelGroup,
-                'Toggle the applied of the filter.\n'
+                'Toggle the applied state of the filter.\n'
                 'Shortcut : <b>Ctrl-T</b>')
+        add_accel_to_button(oLegalButton, "<Ctrl>l", oAccelGroup,
+                'Toggle the display of cards not legal for tournament play.\n'
+                'Shortcut : <b>Ctrl-L</b>')
         add_accel_to_button(oSearchButton, "<Ctrl>f", oAccelGroup,
                 'Open the search dailog for the cards.\n'
                 'Shortcut : <b>Ctrl-F</b>')
@@ -478,7 +552,7 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
             oFilterApplyButton)
 
         oReplacementWin = AutoScrolledWindow(oReplacementView)
-        oReplacementWin.set_size_request(400, 600)
+        oReplacementWin.set_size_request(600, 600)
         oVBox.pack_start(oReplacementWin, True, True)
 
         oFilterButtons = gtk.HBox(spacing=2)
@@ -487,13 +561,15 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
 
         oFilterButtons.pack_start(oFilterDialogButton)
         oFilterButtons.pack_start(oFilterApplyButton)
+        oFilterButtons.pack_start(oLegalButton)
         oFilterButtons.pack_start(oSearchButton)
 
-
         oFilterDialogButton.connect("clicked",
             oReplacementView.run_filter_dialog)
         oFilterApplyButton.connect("toggled",
             oReplacementView.toggle_apply_filter)
+        oLegalButton.connect("toggled",
+            oReplacementView.toggle_show_illegal)
         oSearchButton.connect("clicked",
             oReplacementView.show_search)
 
@@ -527,6 +603,8 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
            We allow the user to select the correct replacements from the
            Abstract Card List.
            """
+        # pylint: disable-msg=R0914
+        # we use lots of variables for clarity
         oUnknownDialog, oHBox = self._create_dialog(sInfo,
                 "The following card names could not be found")
         # pylint: disable-msg=E1101
@@ -542,12 +620,14 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
             aCards = list(oBestGuessFilter.select(AbstractCard))
             if len(aCards) == 1:
                 sBestGuess = aCards[0].name
+                iWeight = pango.WEIGHT_NORMAL
             else:
-                sBestGuess = "No Card"
+                sBestGuess = NO_CARD
+                iWeight = pango.WEIGHT_BOLD
 
             oIter = oModel.append(None)
             # second 1 is the dummy card count
-            oModel.set(oIter, 0, 1, 1, sName, 2, sBestGuess)
+            oModel.set(oIter, 0, 1, 1, sName, 2, sBestGuess, 3, iWeight)
 
         oUnknownDialog.vbox.show_all()
         oAbsCardView.load()
@@ -555,12 +635,14 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
         iResponse = oUnknownDialog.run()
         oUnknownDialog.destroy()
 
+        oAbsCardView.cleanup()
+
         if iResponse == gtk.RESPONSE_OK:
             # For cards marked as replaced, add them to the Holder
             oIter = oModel.get_iter_root()
             while not oIter is None:
                 sName, sNewName = oModel.get(oIter, 1, 2)
-                if sNewName != "No Card":
+                if sNewName != NO_CARD:
                     dUnknownCards[sName] = sNewName
                 oIter = oModel.iter_next(oIter)
             return True
@@ -581,6 +663,7 @@ class GuiLookup(AbstractCardLookup, PhysicalCardLookup, ExpansionLookup):
              gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
 
         aKnownExpansions = list(Expansion.select())
+        aKnownExpansions.sort(key=lambda x: x.name)
 
         oMesgLabel1 = gtk.Label("While importing %s\n"
                 "The following expansions could not be found:\n"
