@@ -13,6 +13,7 @@ from sutekh.gui.CardListFrame import CardListFrame
 from sutekh.gui.PhysicalCardController import PhysicalCardController
 from sutekh.gui.PhysicalCardMenu import PhysicalCardMenu
 
+
 class PhysicalCardFrame(CardListFrame):
     # pylint: disable-msg=R0904
     # gtk.Widget, so many public methods
@@ -34,3 +35,12 @@ class PhysicalCardFrame(CardListFrame):
 
         self._oMenu = PhysicalCardMenu(self, self._oController, oMainWindow)
         self.add_parts()
+
+    def get_menu_name(self):
+        """Get the menu key"""
+        return self._sName
+
+    def cleanup(self):
+        """Cleanup function called before pane is removed by the Main Window"""
+        super(PhysicalCardFrame, self).cleanup()
+        self._oController.cleanup()
