@@ -11,24 +11,25 @@ import gobject
 import unicodedata
 import os
 import zipfile
-import urllib2
 import tempfile
 from sqlobject import SQLObjectNotFound
-from sutekh.core.SutekhObjects import PhysicalCard, PhysicalCardSet, \
-        IAbstractCard, IExpansion
-from sutekh.gui.PluginManager import CardListPlugin
+from sutekh.core.SutekhObjects import IAbstractCard, IExpansion
+from sutekh.gui.PluginManager import SutekhPlugin
 from sutekh.gui.ProgressDialog import ProgressDialog
-from sutekh.gui.CardListView import CardListViewListener
+from sutekh.gui.CardTextView import CardTextViewListener
 from sutekh.gui.BasicFrame import BasicFrame
 from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_buttons, \
         do_complaint_error
 from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
 from sutekh.SutekhUtility import prefs_dir, ensure_dir_exists
+from sutekh.gui.FileOrUrlWidget import FileOrDirOrUrlWidget
+from sutekh.gui.SutekhFileWidget import add_filter
 
 
-_iFORWARD, _iBACKWARD = range(2)
-_iFULL, _iVIEW_FIXED, _iFIT = range(3)
-_tRATIO = (225, 300)
+FORWARD, BACKWARD = range(2)
+FULL, VIEW_FIXED, FIT = range(3)
+RATIO = (225, 300)
+
 
 def _scale_dims(iImageWidth, iImageHeight, iPaneWidth, iPaneHeight):
     """Rescale the image dimension so they fit in the pane, preserving the
@@ -45,6 +46,7 @@ def _scale_dims(iImageWidth, iImageHeight, iPaneWidth, iPaneHeight):
         fDestWidth = iPaneHeight / fImageAspectRatio
     return int(fDestWidth), int(fDestHeight)
 
+
 def _check_file(sFileName):
     """Check if file exists and is readable"""
     bRes = True
@@ -55,8 +57,9 @@ def _check_file(sFileName):
         bRes = False
     return bRes
 
+
 def _unaccent(sCardName):
-    """Remove unicode accents."""
+    """Remove Unicode accents."""
     # Inspired by a post by renaud turned up by google
     # Unicode Normed decomposed form
     sNormed = unicodedata.normalize('NFD',
@@ -64,6 +67,7 @@ def _unaccent(sCardName):
     # Drop non-ascii characters
     return "".join(b for b in sNormed.encode('utf8') if ord(b) < 128)
 
+
 class CardImagePopupMenu(gtk.Menu):
     # pylint: disable-msg=R0904
     # R0904 - can't not trigger these warning with pygtk
@@ -73,30 +77,30 @@ class CardImagePopupMenu(gtk.Menu):
         super(CardImagePopupMenu, self).__init__()
         self.oFrame = oFrame
         self.oZoom = gtk.RadioAction('Zoom', 'Show images at original size',
-                None, None, _iFULL)
+                None, None, FULL)
         self.oZoom.set_group(None)
         self.oViewFixed = gtk.RadioAction('ViewFixed',
-                'Show images at fixed size', None, None, _iVIEW_FIXED)
+                'Show images at fixed size', None, None, VIEW_FIXED)
         self.oViewFixed.set_group(self.oZoom)
         self.oViewFit = gtk.RadioAction('ViewFit', 'Fit images to the pane',
-                None, None, _iFIT)
+                None, None, FIT)
         self.oViewFit.set_group(self.oZoom)
         self.oNext = gtk.Action('NextExp', 'Show next expansion image', None,
                 None)
         self.oPrev = gtk.Action('PrevExp', 'Show previous expansion image',
                 None, None)
 
-        self.oPrev.connect('activate', self.cycle_expansion, _iBACKWARD)
-        self.oNext.connect('activate', self.cycle_expansion, _iFORWARD)
-        self.oViewFit.connect('activate', self.set_zoom, _iFIT)
-        self.oZoom.connect('activate', self.set_zoom, _iFULL)
-        self.oViewFixed.connect('activate', self.set_zoom, _iVIEW_FIXED)
+        self.oPrev.connect('activate', self.cycle_expansion, BACKWARD)
+        self.oNext.connect('activate', self.cycle_expansion, FORWARD)
+        self.oViewFit.connect('activate', self.set_zoom, FIT)
+        self.oZoom.connect('activate', self.set_zoom, FULL)
+        self.oViewFixed.connect('activate', self.set_zoom, VIEW_FIXED)
 
-        if iZoomMode == _iFULL:
+        if iZoomMode == FULL:
             self.oZoom.set_active(True)
-        elif iZoomMode == _iVIEW_FIXED:
+        elif iZoomMode == VIEW_FIXED:
             self.oViewFixed.set_active(True)
-        elif iZoomMode == _iFIT:
+        elif iZoomMode == FIT:
             self.oViewFit.set_active(True)
 
         self.add(self.oViewFit.create_menu_item())
@@ -110,19 +114,18 @@ class CardImagePopupMenu(gtk.Menu):
         self.oNext.set_sensitive(bValue)
         self.oPrev.set_sensitive(bValue)
 
-    # pylint: disable-msg=W0613
-    # oWidget needed by gtk function signature
-    def cycle_expansion(self, oWidget, iDir):
+    def cycle_expansion(self, _oWidget, iDir):
         """Change the expansion as requested."""
-        assert(iDir in [_iBACKWARD, _iFORWARD])
+        assert(iDir in (BACKWARD, FORWARD))
         self.oFrame.do_cycle_expansion(iDir)
 
-    def set_zoom(self, oWidget, iScale):
+    def set_zoom(self, _oWidget, iScale):
         """Change the drawing mode."""
-        assert(iScale in [_iFULL, _iVIEW_FIXED, _iFIT])
+        assert(iScale in (FULL, VIEW_FIXED, FIT))
         self.oFrame.set_zoom_mode(iScale)
 
-class CardImageFrame(BasicFrame, CardListViewListener):
+
+class CardImageFrame(BasicFrame, CardTextViewListener):
     # pylint: disable-msg=R0904, R0902
     # R0904 - can't not trigger these warning with pygtk
     # R0902 - we need to keep quite a lot of internal state
@@ -132,9 +135,11 @@ class CardImageFrame(BasicFrame, CardListViewListener):
        and a Viewport (for scrolling)
        """
 
-    def __init__(self, oMainWindow, oConfigFile):
-        super(CardImageFrame, self).__init__(oMainWindow)
-        self._oConfigFile = oConfigFile
+    sMenuFlag = 'Card Image Frame'
+
+    def __init__(self, oImagePlugin):
+        super(CardImageFrame, self).__init__(oImagePlugin.parent)
+        self._oImagePlugin = oImagePlugin
         oVBox = gtk.VBox(False, 2)
         oBox = gtk.EventBox()
         self.oExpansionLabel = gtk.Label()
@@ -149,26 +154,22 @@ class CardImageFrame(BasicFrame, CardListViewListener):
         oBox.add(self._oImage)
 
         # Enable DnD handling, same as for BasicFrame
-        aDragTargets = [ ('STRING', 0, 0),
-                ('text/plain', 0, 0) ]
-        oBox.drag_dest_set(gtk.DEST_DEFAULT_ALL,
-                aDragTargets,
-                gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
-        oBox.connect('drag-data-received', self.drag_drop_handler)
-        oBox.connect('drag-motion', self.drag_motion)
+        self.set_drag_handler(oBox)
+        self.set_drop_handler(oBox)
         oBox.connect('button-press-event', self.__cycle_expansion)
 
-        self.__sPrefsPath = self._oConfigFile.get_plugin_key('card image path')
+        self.__sPrefsPath = self._oImagePlugin.get_config_item(
+                'card image path')
         if self.__sPrefsPath is None:
             self.__sPrefsPath = os.path.join(prefs_dir('Sutekh'), 'cardimages')
-            self._oConfigFile.set_plugin_key('card image path',
+            self._oImagePlugin.set_config_item('card image path',
                     self.__sPrefsPath)
         self.__bShowExpansions = self.__have_expansions()
         self.__sCurExpansion = ''
         self.__aExpansions = []
         self.__iExpansionPos = 0
         self.__sCardName = ''
-        self.__iZoomMode = _iFIT
+        self.__iZoomMode = FIT
         self._tPaneSize = (0, 0)
 
     type = property(fget=lambda self: "Card Image Frame", doc="Frame Type")
@@ -182,13 +183,20 @@ class CardImageFrame(BasicFrame, CardListViewListener):
         return _check_file(sTestFile)
 
     def __convert_expansion(self, sExpansionName):
-        """Convert the Full Expansion name into the abbrevation needed."""
+        """Convert the Full Expansion name into the abbreviation needed."""
         if sExpansionName == '' or not self.__bShowExpansions:
             return ''
         # pylint: disable-msg=E1101
         # pylint doesn't pick up IExpansion methods correctly
         oExpansion = IExpansion(sExpansionName)
-        return oExpansion.shortname.lower()
+        # special case Anarchs and alastors due to promo hack shortname
+        if oExpansion.name == 'Anarchs and Alastors Storyline':
+            sExpName = oExpansion.name.lower()
+        else:
+            sExpName = oExpansion.shortname.lower()
+        # Normalise for storyline cards
+        sExpName = sExpName.replace(' ', '_').replace("'", '')
+        return sExpName
 
     def __set_expansion_info(self, sCardName):
         """Set the expansion info."""
@@ -201,7 +209,7 @@ class CardImageFrame(BasicFrame, CardListViewListener):
             bHasInfo = False
         if bHasInfo:
             aExp = [oP.expansion.name for oP in oAbsCard.rarity]
-            self.__aExpansions = sorted(list(set(aExp))) # remove duplicates
+            self.__aExpansions = sorted(list(set(aExp)))  # remove duplicates
             self.__iExpansionPos = 0
             self.__sCurExpansion = self.__aExpansions[0]
         else:
@@ -220,19 +228,21 @@ class CardImageFrame(BasicFrame, CardListViewListener):
     def __convert_cardname(self):
         """Convert sCardName to the form used by the card image list"""
         sCurExpansionPath = self.__convert_expansion(self.__sCurExpansion)
-        sFilename = _unaccent(self.__sCardName.lower())
-        if sFilename.find('the ') == 0:
+        sFilename = _unaccent(self.__sCardName)
+        if sFilename.startswith('the '):
             sFilename = sFilename[4:] + 'the'
+        elif sFilename.startswith('an '):
+            sFilename = sFilename[3:] + 'an'
         sFilename = sFilename.replace('(advanced)', 'adv')
         # Should probably do this via translate
-        for sChar in [" ", ".", ",", "'", "(", ")", "-", ":", "!", '"', "/"]:
+        for sChar in (" ", ".", ",", "'", "(", ")", "-", ":", "!", '"', "/"):
             sFilename = sFilename.replace(sChar, '')
         sFilename = sFilename + '.jpg'
         return os.path.join(self.__sPrefsPath, sCurExpansionPath, sFilename)
 
     def __load_image(self, sFullFilename):
         """Load an image into the pane, show broken image if needed"""
-        self._oImage.set_alignment(0.5, 0.5) # Centre image
+        self._oImage.set_alignment(0.5, 0.5)  # Centre image
         try:
             if self.__bShowExpansions:
                 self.oExpansionLabel.set_markup('<i>Image from expansion :'
@@ -242,12 +252,12 @@ class CardImageFrame(BasicFrame, CardListViewListener):
                 # pylint doesn't pick up allocation methods correctly
                 iHeightOffset = self.oExpansionLabel.allocation.height + 2
             else:
-                self.oExpansionLabel.hide() # config chanes can cause this
+                self.oExpansionLabel.hide()  # config chanes can cause this
                 iHeightOffset = 0
             oPixbuf = gtk.gdk.pixbuf_new_from_file(sFullFilename)
             iWidth = oPixbuf.get_width()
             iHeight = oPixbuf.get_height()
-            if self.__iZoomMode == _iFIT:
+            if self.__iZoomMode == FIT:
                 # Need to fix aspect ratios
                 iPaneHeight = self._oView.get_vadjustment().page_size - \
                         iHeightOffset
@@ -261,9 +271,9 @@ class CardImageFrame(BasicFrame, CardListViewListener):
                         iDestWidth, iDestHeight, gtk.gdk.INTERP_HYPER))
                     self._tPaneSize = (self._oView.get_hadjustment().page_size,
                             self._oView.get_vadjustment().page_size)
-            elif self.__iZoomMode == _iVIEW_FIXED:
+            elif self.__iZoomMode == VIEW_FIXED:
                 iDestWidth, iDestHeight = _scale_dims(iWidth, iHeight,
-                        _tRATIO[0], _tRATIO[1])
+                        RATIO[0], RATIO[1])
                 self._oImage.set_from_pixbuf(oPixbuf.scale_simple(iDestWidth,
                     iDestHeight, gtk.gdk.INTERP_HYPER))
             else:
@@ -289,11 +299,17 @@ class CardImageFrame(BasicFrame, CardListViewListener):
     def update_config_path(self, sNewPath):
         """Update the path we use to search for expansions."""
         self.__sPrefsPath = sNewPath
-        self._oConfigFile.set_plugin_key('card image path', sNewPath)
+        self._oImagePlugin.set_config_item('card image path', sNewPath)
         self.__bShowExpansions = self.__have_expansions()
 
-    def set_card_text(self, sCardName, sExpansionName=''):
+    def set_card_text(self, oPhysCard):
         """Set the image in response to a set card name event."""
+        if not oPhysCard:
+            return
+        sCardName = oPhysCard.abstractCard.canonicalName
+        sExpansionName = ''
+        if oPhysCard.expansion:
+            sExpansionName = oPhysCard.expansion.name
         if sCardName != self.__sCardName:
             self.__set_expansion_info(sCardName)
             self.__sCardName = sCardName
@@ -302,22 +318,33 @@ class CardImageFrame(BasicFrame, CardListViewListener):
                 # Honour expansion from set_card_text
                 self.__sCurExpansion = sExpansionName
                 self.__iExpansionPos = self.__aExpansions.index(sExpansionName)
-            elif self.__sCurExpansion == '' or \
-                    self.__sCurExpansion not in self.__aExpansions:
-                # Set self.__sCurExpansion to a valid value
-                self.__sCurExpansion = self.__aExpansions[0]
+            else:
+                # Set self.__sCurExpansion to an existing image, if possible
                 self.__iExpansionPos = 0
+                bFound = False
+                while not bFound and \
+                        self.__iExpansionPos < len(self.__aExpansions):
+                    self.__sCurExpansion = \
+                            self.__aExpansions[self.__iExpansionPos]
+                    sFullFilename = self.__convert_cardname()
+                    if _check_file(sFullFilename):
+                        bFound = True
+                    else:
+                        self.__iExpansionPos += 1
+                if not bFound:
+                    self.__sCurExpansion = self.__aExpansions[0]
+                    self.__iExpansionPos = 0
         self.__redraw(False)
 
     def do_cycle_expansion(self, iDir):
         """Change the expansion image to a different one in the list."""
         if len(self.__aExpansions) < 2 or not self.__bShowExpansions:
-            return # nothing to scroll through
-        if iDir == _iFORWARD:
+            return  # nothing to scroll through
+        if iDir == FORWARD:
             self.__iExpansionPos += 1
             if self.__iExpansionPos >= len(self.__aExpansions):
                 self.__iExpansionPos = 0
-        elif iDir == _iBACKWARD:
+        elif iDir == BACKWARD:
             self.__iExpansionPos -= 1
             if self.__iExpansionPos < 0:
                 self.__iExpansionPos = len(self.__aExpansions) - 1
@@ -329,14 +356,12 @@ class CardImageFrame(BasicFrame, CardListViewListener):
         self.__iZoomMode = iScale
         self.__redraw(False)
 
-    # pylint: disable-msg=W0613
-    # oWidget needed by gtk function signature
-    def __cycle_expansion(self, oWidget, oEvent):
+    def __cycle_expansion(self, _oWidget, oEvent):
         """On a button click, move to the next expansion."""
         if oEvent.type != gtk.gdk.BUTTON_PRESS:
-            return True # don't jump twice on double or triple clicks
+            return True  # don't jump twice on double or triple clicks
         if oEvent.button == 1:
-            self.do_cycle_expansion(_iFORWARD)
+            self.do_cycle_expansion(FORWARD)
         elif oEvent.button == 3:
             # Do context menu
             oPopupMenu = CardImagePopupMenu(self, self.__iZoomMode)
@@ -345,23 +370,31 @@ class CardImageFrame(BasicFrame, CardListViewListener):
             oPopupMenu.popup(None, None, None, oEvent.button, oEvent.time)
         return True
 
-    # W0613 oAdjust also required by signature
-    def _pane_adjust(self, oAdjust):
+    def _pane_adjust(self, _oAdjust):
         """Redraw the image if needed when the pane size changes."""
-        if self.__iZoomMode == _iFIT:
+        if self.__iZoomMode == FIT:
             tCurSize = (self._oView.get_hadjustment().page_size,
                 self._oView.get_vadjustment().page_size)
             if tCurSize[0] != self._tPaneSize[0] or \
                     tCurSize[1] != self._tPaneSize[1]:
                 self.__redraw(True)
 
-# pylint: disable-msg=R0904
-# R0904 - gtk Widget, so has many public methods
+    def get_menu_name(self):
+        """Return the menu key"""
+        return self.sMenuFlag
+
+
 class ImageConfigDialog(SutekhDialog):
+    # pylint: disable-msg=R0904
+    # R0904 - gtk Widget, so has many public methods
     """Dialog for configuring the Image plugin."""
-    def __init__(self, oParent, bFirstTime=False):
+
+    sDefaultUrl = 'http://csillagmag.hu/upload/pictures.zip'
+
+    def __init__(self, oImagePlugin, bFirstTime=False):
         super(ImageConfigDialog, self).__init__('Configure Card Images Plugin',
-                oParent, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                oImagePlugin.parent,
+                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                 (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL,
                     gtk.RESPONSE_CANCEL))
         oDescLabel = gtk.Label()
@@ -372,113 +405,63 @@ class ImageConfigDialog(SutekhDialog):
             oDescLabel.set_markup('<b>Choose how to configure the cardimages '
                     'plugin</b>\nChoose cancel to skip configuring the '
                     'images plugin\nYou will not be prompted again')
-        self.oChoiceDownload = gtk.RadioButton(
-                label='Download and install cardimages from feldb.extra.hu')
-        self.oChoiceLocalCopy = gtk.RadioButton(self.oChoiceDownload,
-                label='Install images from a local zip file')
-        self.oChoiceNewDir = gtk.RadioButton(self.oChoiceDownload,
-                label='Choose a directory containing the images')
-        oChoiceBox = gtk.VBox(False, 2)
-        self.oDirChoiceWidget = gtk.FileChooserWidget(
-                action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
-        self.oFileChoiceWidget = gtk.FileChooserWidget(
-                action=gtk.FILE_CHOOSER_ACTION_OPEN)
-        self.oChoiceDownload.connect('toggled', self._radio_toggled,
-                oChoiceBox, None)
-        self.oChoiceLocalCopy.connect('toggled', self._radio_toggled,
-                oChoiceBox, self.oFileChoiceWidget)
-        self.oChoiceNewDir.connect('toggled', self._radio_toggled, oChoiceBox,
-                self.oDirChoiceWidget)
-
-        self.oChoiceDownload.set_active(True)
+        sDefaultDir = oImagePlugin.get_config_item('card image path')
+        self.oChoice = FileOrDirOrUrlWidget(oImagePlugin.parent,
+                "Choose location for "
+                "images file", "Choose image directory", sDefaultDir,
+                {'csillagbolcselet.hu': self.sDefaultUrl})
+        add_filter(self.oChoice, 'Zip Files', ['*.zip', '*.ZIP'])
         # pylint: disable-msg=E1101
         # pylint doesn't pick up vbox methods correctly
         self.vbox.pack_start(oDescLabel, False, False)
-        self.vbox.pack_start(self.oChoiceDownload, False, False)
-        self.vbox.pack_start(self.oChoiceLocalCopy, False, False)
-        self.vbox.pack_start(self.oChoiceNewDir, False, False)
-        self.vbox.pack_start(oChoiceBox)
-        self.set_size_request(600, 600)
+        self.vbox.pack_start(self.oChoice, False, False)
+        self.set_size_request(400, 200)
 
-        #oDialog.vbox.pack_start(oFileWidget)
-        sPrefsPath = oParent.config_file.get_plugin_key('card image path')
-        if sPrefsPath is not None:
-            if os.path.exists(sPrefsPath) and os.path.isdir(sPrefsPath):
-                # File widget doesn't like being pointed at non-dirs
-                # when in SELECT_FOLDER mode
-                self.oDirChoiceWidget.set_current_folder(sPrefsPath)
-        oZipFilter = gtk.FileFilter()
-        oZipFilter.add_pattern('*.zip')
-        oZipFilter.add_pattern('*.ZIP')
-        self.oFileChoiceWidget.add_filter(oZipFilter)
         self.show_all()
 
-    def _radio_toggled(self, oRadioButton, oChoiceBox, oFileWidget):
-        """Display the correct file widget when radio buttons change"""
-        if oRadioButton.get_active():
-            for oChild in oChoiceBox.get_children():
-                oChoiceBox.remove(oChild)
-            if oFileWidget:
-                oChoiceBox.pack_start(oFileWidget)
-            self.show_all()
+    def get_data(self):
+        """Get the results of the users choice."""
 
-    def get_path(self):
-        """Get the path from oDirChoiceWidget."""
-        return self.oDirChoiceWidget.get_filename()
+        sFile, _bUrl, bDir = self.oChoice.get_file_or_dir_or_url()
+        if bDir:
+            # Just return the name the user chose
+            return sFile, True
+        if sFile:
+            oOutFile = tempfile.TemporaryFile()
+            self.oChoice.get_binary_data(oOutFile)
+            return oOutFile, False
+        return None, None
 
-    def get_file(self):
-        """Get the file from oFileChoiceWidget."""
-        return self.oFileChoiceWidget.get_filename()
 
-    def get_choice(self):
-        """Get which of the RadioButtons was active."""
-        if self.oChoiceDownload.get_active():
-            return 'Download'
-        elif self.oChoiceLocalCopy.get_active():
-            return 'Local copy'
-        else:
-            return 'New directory'
-
-class CardImagePlugin(CardListPlugin):
+class CardImagePlugin(SutekhPlugin):
     """Plugin providing access to CardImageFrame."""
-    dTableVersions = {PhysicalCardSet : [5]}
-    aModelsSupported = ["MainWindow"]
-    aListenViews = [PhysicalCardSet,
-            PhysicalCard]
+    dTableVersions = {}
+    aModelsSupported = ("MainWindow",)
 
-    oImageFrame = None
+    dGlobalConfig = {
+        'card image path': 'string(default=None)',
+    }
 
-    _sMenuFlag = 'Card Image Frame'
+    _sMenuFlag = CardImageFrame.sMenuFlag
 
     # pylint: disable-msg=W0142
     # ** magic OK here
-    def __init__(self, *aArgs, **kwargs):
-        super(CardImagePlugin, self).__init__(*aArgs, **kwargs)
-        # Add listeners to the appropriate views
-        if not self.image_frame:
-            self.init_image_frame(self)
-        if self._cModelType in self.aListenViews:
-            self.view.add_listener(self.image_frame)
+    def __init__(self, *args, **kwargs):
+        super(CardImagePlugin, self).__init__(*args, **kwargs)
+        self.oImageFrame = None
         self._oReplaceItem = None
         self._oAddItem = None
         self._oConfigMenuItem = None
 
-    image_frame = property(fget=lambda self: self.get_image_frame(),
+    image_frame = property(fget=lambda self: self.oImageFrame,
             doc="The image frame")
 
-    @classmethod
-    def get_image_frame(cls):
-        """Return the global ImageFrame."""
-        return cls.oImageFrame
-
-    @classmethod
-    def init_image_frame(cls, oCur):
-        """Setup the global image frame."""
-        if not cls.oImageFrame:
-            cls.oImageFrame = CardImageFrame(oCur.parent,
-                    oCur.parent.config_file)
-            cls.oImageFrame.set_title(cls._sMenuFlag)
-            cls.oImageFrame.add_parts()
+    def init_image_frame(self):
+        """Setup the image frame."""
+        if not self.oImageFrame:
+            self.oImageFrame = CardImageFrame(self)
+            self.oImageFrame.set_title(self._sMenuFlag)
+            self.oImageFrame.add_parts()
 
     def get_menu_item(self):
         """Overrides method from base class.
@@ -487,6 +470,9 @@ class CardImagePlugin(CardListPlugin):
            """
         if not self.check_versions() or not self.check_model_type():
             return None
+        self.init_image_frame()
+        # Add listener
+        self.parent.card_text_pane.view.add_listener(self.oImageFrame)
         self._oReplaceItem = gtk.MenuItem("Replace with Card Image Frame")
         self._oReplaceItem.connect("activate", self.replace_pane)
 
@@ -494,33 +480,31 @@ class CardImagePlugin(CardListPlugin):
         self._oAddItem.connect("activate", self.add_pane)
         self.parent.add_to_menu_list('Card Image Frame',
                 self.add_image_frame_active)
-        self._oConfigMenuItem = gtk.MenuItem("Configure Card Images Plugin")
+        self._oConfigMenuItem = gtk.MenuItem(
+                "Download or Configure Card Images")
         self._oConfigMenuItem.connect("activate", self.config_activate)
         if not self.image_frame.check_images():
             # Don't allow the menu option if we can't find the images
             self.add_image_frame_active(False)
-        return [('Plugins', self._oConfigMenuItem),
+        return [('Data Downloads', self._oConfigMenuItem),
                 ('Add Pane', self._oAddItem),
                 ('Replace Pane', self._oReplaceItem)]
 
     def setup(self):
         """Prompt the user to download/setup images the first time"""
-        sPrefsPath = self.parent.config_file.get_plugin_key('card image path')
+        sPrefsPath = self.get_config_item('card image path')
         if not os.path.exists(sPrefsPath):
             # Looks like the first time
-            oDialog = ImageConfigDialog(self.parent, True)
+            oDialog = ImageConfigDialog(self, True)
             self.handle_response(oDialog)
             # Path may have been changed, so we need to requery config file
-            sPrefsPath = self.parent.config_file.get_plugin_key(
-                    'card image path')
+            sPrefsPath = self.get_config_item('card image path')
             # Don't get called next time
             ensure_dir_exists(sPrefsPath)
 
-    # pylint: disable-msg=W0613
-    # oMenuWidget needed by gtk function signature
-    def config_activate(self, oMenuWidget):
+    def config_activate(self, _oMenuWidget):
         """Configure the plugin dialog."""
-        oDialog = ImageConfigDialog(self.parent, False)
+        oDialog = ImageConfigDialog(self, False)
         self.handle_response(oDialog)
 
     def handle_response(self, oDialog):
@@ -528,84 +512,41 @@ class CardImagePlugin(CardListPlugin):
         iResponse = oDialog.run()
         bActivateMenu = False
         if iResponse == gtk.RESPONSE_OK:
-            sChoice = oDialog.get_choice()
-            if sChoice == 'Download':
-                if self._download_file():
-                    bActivateMenu = True
-                else:
-                    do_complaint_error('Unable to successfully download'
-                            ' and install zipfile')
-            elif sChoice == 'Local copy':
-                sFileName = oDialog.get_file()
-                if self._unzip_file(sFileName):
-                    bActivateMenu = True
-                else:
-                    do_complaint_error('Unable to successfully install '
-                            ' zipfile %s' % sFileName)
-            else:
+            oFile, bDir = oDialog.get_data()
+            if bDir:
                 # New directory
-                sTestPath = oDialog.get_path()
-                if self._accept_path(sTestPath):
+                if self._accept_path(oFile):
                     # Update preferences
-                    self.image_frame.update_config_path(sTestPath)
+                    self.image_frame.update_config_path(oFile)
                     bActivateMenu = True
+            elif oFile:
+                if self._unzip_file(oFile):
+                    bActivateMenu = True
+                else:
+                    do_complaint_error('Unable to successfully unzip data')
+                oFile.close()  # clean up temp file
+            else:
+                # Unable to get data
+                do_complaint_error('Unable to configure card images plugin')
         if bActivateMenu:
             # Update the menu display if needed
-            if self._sMenuFlag not in self.parent.dOpenFrames.values():
+            if not self.parent.is_open_by_menu_name(self._sMenuFlag):
                 # Pane is not open, so try to enable menu
                 self.add_image_frame_active(True)
         # get rid of the dialog
         oDialog.destroy()
 
-    def _download_file(self):
-        """Download a zip file containing the images."""
-        # Download and install http://feldb.extra.hu/pictures.zip
-        sUrl = 'http://feldb.extra.hu/pictures.zip'
-        oFile = urllib2.urlopen(sUrl)
-        sDownloadDir = prefs_dir('Sutekh')
-        ensure_dir_exists(sDownloadDir)
-        fTemp = tempfile.TemporaryFile()
-        sLength = oFile.info().getheader('Content-Length')
-        if sLength:
-            iLength = int(sLength)
-            oProgress = ProgressDialog()
-            oProgress.set_description('Download progress')
-            iTotal = 0
-            bCont = True
-            while bCont:
-                oInf = oFile.read(10000)
-                iTotal += 10000
-                oProgress.update_bar(float(iTotal) / iLength)
-                if oInf:
-                    fTemp.write(oInf)
-                else:
-                    bCont = False
-                    oProgress.destroy()
-        else:
-            # Just try and download
-            fTemp.write(oFile.read())
-        try:
-            oZipFile = zipfile.ZipFile(fTemp, 'r')
-            bRet = self._unzip_heart(oZipFile)
-        except zipfile.BadZipfile:
-            bRet = False
-        # cleanup file
-        fTemp.close()
-        return bRet
-
-    def _unzip_file(self, sFileName):
+    def _unzip_file(self, oFile):
         """Unzip a file containing the images."""
-        if _check_file(sFileName):
-            try:
-                oZipFile = zipfile.ZipFile(sFileName, 'r')
-            except zipfile.BadZipfile:
-                return False
-            return self._unzip_heart(oZipFile)
-        return False
+        try:
+            oZipFile = zipfile.ZipFile(oFile)
+        except zipfile.BadZipfile:
+            return False
+        return self._unzip_heart(oZipFile)
 
     def _unzip_heart(self, oZipFile):
         """Heavy lifting of unzipping a file"""
-        sPrefsPath = self.parent.config_file.get_plugin_key('card image path')
+        sPrefsPath = self.get_config_item('card image path')
         ensure_dir_exists(sPrefsPath)
         iNumber = len(oZipFile.infolist())
         if iNumber < 300:
@@ -624,7 +565,7 @@ class CardImagePlugin(CardListPlugin):
                 continue
             # Should I also test for cardimages\ ?
             sFileName = os.path.join(sPrefsPath,
-                    oItem.filename.replace('cardimages/',''))
+                    oItem.filename.replace('cardimages/', ''))
             sFileName = sFileName.replace('/', os.path.sep)
             sDir = os.path.dirname(sFileName)
             ensure_dir_exists(sDir)
@@ -650,7 +591,7 @@ class CardImagePlugin(CardListPlugin):
                     # Treat as cancelling
                     return False
             return True
-        return False # No path, can't be OK
+        return False  # No path, can't be OK
 
     def add_image_frame_active(self, bValue):
         """Toggle the sensitivity of the menu item."""
@@ -665,28 +606,24 @@ class CardImagePlugin(CardListPlugin):
     def get_frame_from_config(self, sType):
         """Add the frame if it's been saved in the config file."""
         if sType == self._sMenuFlag:
-            return (self.image_frame, self._sMenuFlag)
+            return self.image_frame
         else:
             return None
 
-    # pylint: disable-msg=W0613
-    # oWidget needed by gtk function signature
-    def replace_pane(self, oWidget):
+    def replace_pane(self, _oWidget):
         """Handle replacing a frame to the main window if required"""
-        if self._sMenuFlag not in self.parent.dOpenFrames.values():
+        if not self.parent.is_open_by_menu_name(self._sMenuFlag):
             oNewPane = self.parent.focussed_pane
             if oNewPane:
-                self.parent.replace_frame(oNewPane, self.image_frame,
-                        self._sMenuFlag)
+                self.image_frame.set_unique_id()
+                self.parent.replace_frame(oNewPane, self.image_frame)
 
-    def add_pane(self, oWidget):
+    def add_pane(self, _oWidget):
         """Handle adding the frame to the main window if required"""
-        if self._sMenuFlag not in self.parent.dOpenFrames.values():
+        if not self.parent.is_open_by_menu_name(self._sMenuFlag):
             oNewPane = self.parent.add_pane_end()
-            self.parent.replace_frame(oNewPane, self.image_frame,
-                    self._sMenuFlag)
+            self.image_frame.set_unique_id()
+            self.parent.replace_frame(oNewPane, self.image_frame)
 
 
-# pylint: disable-msg=C0103
-# shut up complaint about the name
 plugin = CardImagePlugin
