@@ -8,16 +8,16 @@
 
 import gtk
 from sutekh.core.SutekhObjects import PhysicalCard, IPhysicalCard, \
-        PhysicalCardSet, IAbstractCard, IExpansion
+        PhysicalCardSet, IAbstractCard, IPhysicalCardSet
 from sutekh.core.Filters import PhysicalCardSetFilter
-from sutekh.gui.PluginManager import CardListPlugin
+from sutekh.gui.PluginManager import SutekhPlugin
 from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error
-from sutekh.gui.ScrolledList import ScrolledList
+from sutekh.gui.CardSetsListView import CardSetsListView
 from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
-from sutekh.gui.CreateCardSetDialog import CreateCardSetDialog
-from sutekh.gui.CardSetManagementController import update_card_set
+from sutekh.gui.GuiCardSetFunctions import create_card_set
 
-def _get_card_set_list(aCardSetNames, bUseExpansions):
+
+def _get_card_set_list(aCardSetNames, bIgnoreExpansions):
     """Get the differences and common cards for the card sets."""
     # Only compare abstract cards
     dFullCardList = {}
@@ -28,125 +28,130 @@ def _get_card_set_list(aCardSetNames, bUseExpansions):
             # pylint: disable-msg=E1101
             # pylint doesn't see IAbstractCard methods
             oAbsCard = IAbstractCard(oCard)
-            if bUseExpansions:
-                if oCard.expansion:
-                    tKey = (oAbsCard.name, oCard.expansion.name)
-                else:
-                    tKey = (oAbsCard.name, 'Unspecified Expansion')
+            if bIgnoreExpansions:
+                oKey = (oAbsCard, oAbsCard.name, None)
             else:
-                tKey = (oAbsCard.name, None)
-            dFullCardList.setdefault(tKey, {aCardSetNames[0] : 0,
-                aCardSetNames[1] : 0})
-            dFullCardList[tKey][sCardSetName] += 1
-    dDifferences = { aCardSetNames[0] : [], aCardSetNames[1] : [] }
-    aCommon = []
+                if oCard.expansion:
+                    oKey = (oCard, oAbsCard.name, oCard.expansion.name)
+                else:
+                    oKey = (oAbsCard, oAbsCard.name, 'Unspecified Expansion')
+            dFullCardList.setdefault(oKey, {aCardSetNames[0]: 0,
+                aCardSetNames[1]: 0})
+            dFullCardList[oKey][sCardSetName] += 1
+    dDifferences = {aCardSetNames[0]: {}, aCardSetNames[1]: {}}
+    dCommon = {}
     for tCardInfo in dFullCardList:
         iDiff = dFullCardList[tCardInfo][aCardSetNames[0]] - \
                 dFullCardList[tCardInfo][aCardSetNames[1]]
         iCommon = min(dFullCardList[tCardInfo][aCardSetNames[0]],
                 dFullCardList[tCardInfo][aCardSetNames[1]])
         if iDiff > 0:
-            dDifferences[aCardSetNames[0]].append((tCardInfo[0], tCardInfo[1],
-                iDiff))
+            dDifferences[aCardSetNames[0]][tCardInfo[0]] = (tCardInfo[1],
+                    tCardInfo[2], iDiff)
         elif iDiff < 0:
-            dDifferences[aCardSetNames[1]].append((tCardInfo[0], tCardInfo[1],
-                abs(iDiff)))
+            dDifferences[aCardSetNames[1]][tCardInfo[0]] = (tCardInfo[1],
+                    tCardInfo[2], abs(iDiff))
         if iCommon > 0:
-            aCommon.append((tCardInfo[0], tCardInfo[1], iCommon))
-    return (dDifferences, aCommon)
+            dCommon[tCardInfo[0]] = (tCardInfo[1], tCardInfo[2], iCommon)
+    return (dDifferences, dCommon)
 
 
-class CardSetCompare(CardListPlugin):
+class CardSetCompare(SutekhPlugin):
     """Compare Two Card Sets
 
        Display a gtk.Notebook containing tabs for common cards, and cards
        only in each of the card sets.
        """
-    dTableVersions = {PhysicalCardSet : [5]}
-    aModelsSupported = [PhysicalCardSet]
+    dTableVersions = {PhysicalCardSet: (5, 6)}
+    aModelsSupported = (PhysicalCardSet,)
 
     def get_menu_item(self):
-        """Register on the 'Plugins' menu."""
+        """Register on the 'Analyze' menu."""
         if not self.check_versions() or not self.check_model_type():
             return None
         oCompare = gtk.MenuItem("Compare with another Card Set")
         oCompare.connect("activate", self.activate)
-        return ('Plugins', oCompare)
+        return ('Analyze', oCompare)
 
-    # pylint: disable-msg=W0613
-    # oWidget required by gtk function signature
-    def activate(self, oWidget):
+    def activate(self, _oWidget):
         """Create the dialog for choosing the second card set."""
         oDlg = SutekhDialog("Choose Card Set to Compare with", self.parent,
                 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                 (gtk.STOCK_OK, gtk.RESPONSE_OK,
                     gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
-        oSelect = PhysicalCardSet.select().orderBy('name')
-        oCSList = ScrolledList('Card Sets')
-        oCSList.set_select_single()
+        oCSView = CardSetsListView(None, oDlg)
         # pylint: disable-msg=E1101
         # pylint misses vbox methods
-        oDlg.vbox.pack_start(oCSList)
-        oUseExpansions = gtk.CheckButton("Consider Expansions as well")
-        oDlg.vbox.pack_start(oUseExpansions)
-        oCSList.set_size_request(150, 300)
-        aVals = [oCS.name for oCS in oSelect if oCS.name != self.view.sSetName]
-        oCSList.fill_list(aVals)
-        oDlg.connect("response", self.handle_response, oCSList, oUseExpansions)
+        oCSView.set_select_single()
+        oCSView.exclude_set(self.view.sSetName)
+        oDlg.vbox.pack_start(AutoScrolledWindow(oCSView), expand=True)
+        oIgnoreExpansions = gtk.CheckButton("Ignore Expansions")
+        oDlg.vbox.pack_start(oIgnoreExpansions)
+        oCSView.set_size_request(450, 300)
+        oDlg.connect("response", self.handle_response, oCSView,
+                oIgnoreExpansions)
         oDlg.show_all()
         oDlg.run()
         oDlg.destroy()
 
-    def handle_response(self, oWidget, iResponse, oCSList, oUseExpansions):
+    def handle_response(self, _oWidget, iResponse, oCSView, oIgnoreExpansions):
         """Handle response from the dialog."""
-        if iResponse ==  gtk.RESPONSE_OK:
+        if iResponse == gtk.RESPONSE_OK:
             aCardSetNames = [self.view.sSetName]
-            aCardSetNames.extend(oCSList.get_selection())
-            if len(aCardSetNames) != 2:
+            sSet = oCSView.get_selected_card_set()
+            if not sSet:
                 do_complaint_error("Need to choose a card set to compare to")
             else:
-                self.comp_card_sets(aCardSetNames, oUseExpansions.get_active())
+                aCardSetNames.append(sSet)
+                self.comp_card_sets(aCardSetNames,
+                        oIgnoreExpansions.get_active())
 
-    def comp_card_sets(self, aCardSetNames, bUseExpansions):
+    def comp_card_sets(self, aCardSetNames, bIgnoreExpansions):
         """Display the results of comparing the card sets."""
-        def format_list(aList, sColor):
+        def format_list(dCardInfo, sColor):
             """Format the list of cards for display."""
             oLabel = gtk.Label()
             oAlign = gtk.Alignment()
             oAlign.add(oLabel)
-            oAlign.set_padding(0, 0, 5, 0) # offset a little from the left edge
-            if len(aList) > 0:
-                sContents = ""
+            oAlign.set_padding(0, 0, 5, 0)  # offset from the left edge
+            oGroupedCards = self.model.groupby(dCardInfo, IAbstractCard)
+            sContents = ""
+            for sGroup, oGroupIter in oGroupedCards:
+                aList = [dCardInfo[x] for x in oGroupIter]
+                iTot = sum([x[2] for x in aList])
                 aList.sort()
+                sContents += '<span foreground="blue">%s - %d cards' \
+                        '</span>\n' % (sGroup, iTot)
                 for sCardName, sExpansion, iCount in aList:
-                    sContents += '%(num)d X <span foreground = "%(color)s">' \
-                            '%(name)s' % {
-                                    'num' : iCount,
-                                    'color' : sColor,
-                                    'name' : sCardName,
+                    sContents += u'\t%(num)d \u00D7 <span foreground =' \
+                            ' "%(color)s"> %(name)s' % {
+                                    'num': iCount,
+                                    'color': sColor,
+                                    'name': sCardName,
                                     }
                     if sExpansion:
                         sContents += " (%s)</span>\n" % sExpansion
                     else:
                         sContents += "</span>\n"
+            if sContents:
                 oLabel.set_markup(sContents)
             else:
                 oLabel.set_text('No Cards')
             return oAlign
 
-        def make_page(oList, aCardData):
+        def make_page(oList, dCardData):
             """Setup the list + button for the notebook"""
             oPage = gtk.VBox(False)
             oPage.pack_start(AutoScrolledWindow(oList, True), True)
-            if aCardData:
+            if dCardData:
                 oButton = gtk.Button('Create Card Set from this list')
                 oButton.connect('clicked', self.create_card_set,
-                        aCardData)
+                        dCardData)
                 oPage.pack_start(oButton, False)
             return oPage
 
-        (dDifferences, aCommon) = _get_card_set_list(aCardSetNames,
-                bUseExpansions)
+        (dDifferences, dCommon) = _get_card_set_list(aCardSetNames,
+                bIgnoreExpansions)
         oResultDlg = SutekhDialog("Card Comparison", self.parent,
                 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                 (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
@@ -156,16 +161,16 @@ class CardSetCompare(CardListPlugin):
         oHeading = gtk.Label()
         sTabText = 'Common Cards'
         oHeading.set_markup('<span foreground = "blue">%s</span>' % sTabText)
-        oComm = format_list(aCommon, 'green')
-        oPage = make_page(oComm, aCommon)
+        oComm = format_list(dCommon, 'green')
+        oPage = make_page(oComm, dCommon)
         oNotebook.append_page_menu(oPage, oHeading, gtk.Label(sTabText))
         oHeading = gtk.Label()
-        sTabText ='Cards only in %s' % aCardSetNames[0]
+        sTabText = 'Cards only in %s' % self.escape(aCardSetNames[0])
         oHeading.set_markup('<span foreground = "red">%s</span>' % sTabText)
         oDiff1 = format_list(dDifferences[aCardSetNames[0]], 'red')
         oPage = make_page(oDiff1, dDifferences[aCardSetNames[0]])
         oNotebook.append_page_menu(oPage, oHeading, gtk.Label(sTabText))
-        sTabText ='Cards only in %s' % aCardSetNames[1]
+        sTabText = 'Cards only in %s' % self.escape(aCardSetNames[1])
         oHeading = gtk.Label()
         oHeading.set_markup('<span foreground = "red">%s</span>' % sTabText)
         oDiff2 = format_list(dDifferences[aCardSetNames[1]], 'red')
@@ -179,34 +184,25 @@ class CardSetCompare(CardListPlugin):
         oResultDlg.run()
         oResultDlg.destroy()
 
-    def create_card_set(self, oButton, aCardData):
+    def create_card_set(self, _oButton, dCardData):
         """Create a card set from the card list"""
-        oDlg = CreateCardSetDialog(self.parent)
-        oDlg.run()
-        sCSName = oDlg.get_name()
+        sCSName = create_card_set(self.parent)
         if not sCSName:
-            return
-        if PhysicalCardSet.selectBy(name=sCSName).count() != 0:
-            do_complaint_error("Card Set %s already exists."
-                    % sCSName)
-            return
-        oCardSet = PhysicalCardSet(name=sCSName)
-        for sCardName, sExpansionName, iCnt in aCardData:
-            if sExpansionName and sExpansionName != 'Unspecified Expansion':
-                oExpansion = IExpansion(sExpansionName)
+            return  # User cancelled, so skip out
+        oCardSet = IPhysicalCardSet(sCSName)
+        for oCard in dCardData:
+            _sCardName, sExpansionName, iCnt = dCardData[oCard]
+            if not sExpansionName:
+                # Dealing with abstract cards in the list
+                oPhysCard = IPhysicalCard((oCard, None))
             else:
-                oExpansion = None
-            oCard = IAbstractCard(sCardName)
-            oPhysCard = IPhysicalCard((oCard, oExpansion))
-            # pylint: disable-msg=W0612, E1101
-            # W0612 - iNum is just loop counter
+                # Dealing with pysical cards in the list
+                oPhysCard = oCard
+            # pylint: disable-msg=E1101
             # E1101 - sqlobject confuses pylint
-            for iNum in range(iCnt):
+            for _iNum in range(iCnt):
                 oCardSet.addPhysicalCard(oPhysCard)
-        update_card_set(oCardSet, oDlg, self.parent, None)
         self.open_cs(sCSName)
 
 
-# pylint: disable-msg=C0103
-# accept plugin name
 plugin = CardSetCompare
