@@ -13,11 +13,12 @@ from sutekh.core.SutekhObjects import PhysicalCard, \
                                       PhysicalCardSet, \
                                       IPhysicalCard
 from sutekh.core.CardListTabulator import CardListTabulator
-from sutekh.gui.PluginManager import CardListPlugin
+from sutekh.gui.PluginManager import SutekhPlugin
 from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
 from sutekh.gui.SutekhDialog import SutekhDialog, do_complaint_error
 
-class ClusterCardList(CardListPlugin):
+
+class ClusterCardList(SutekhPlugin):
     """Plugin that attempts to find clusters in the card list.
 
        Allows the user to choose various clustering parameters, such as
@@ -27,17 +28,17 @@ class ClusterCardList(CardListPlugin):
     # we use a lot of attributes to keep track of data & gui aspects
 
     dTableVersions = {}
-    aModelsSupported = [PhysicalCard, PhysicalCardSet]
+    aModelsSupported = (PhysicalCard, PhysicalCardSet)
 
     # pylint: disable-msg=W0142
     # ** magic OK
-    def __init__(self, *aArgs, **kwargs):
-        super(ClusterCardList, self).__init__(*aArgs, **kwargs)
+    def __init__(self, *args, **kwargs):
+        super(ClusterCardList, self).__init__(*args, **kwargs)
         # pylint: disable-msg=C0103
         # fMakeCardFromCluster triggers on length, but we like the name
         if not self.model:
             self._fMakeCardSetFromCluster = None
-        self._fMakeCardSetFromCluster = self.make_pcs_from_cluster
+        self._fMakeCardSetFromCluster = self._make_pcs_from_cluster
         self._dPropButtons = {}
         self._dGroups = {}
         self._oResultsVbox = None
@@ -52,16 +53,14 @@ class ClusterCardList(CardListPlugin):
     # pylint: enable-msg=W0142
 
     def get_menu_item(self):
-        """Register on the 'Plugins' menu."""
+        """Register on the 'Analyze' menu."""
         if not self.check_versions() or not self.check_model_type():
             return None
         oCluster = gtk.MenuItem("Cluster Cards")
         oCluster.connect("activate", self.activate)
-        return ('Plugins', oCluster)
+        return ('Analyze', oCluster)
 
-    # pylint: disable-msg=W0613
-    # oWidget required by function signature
-    def activate(self, oWidget):
+    def activate(self, _oWidget):
         """In response to the menu, create the correct dialog."""
         oDlg = self.make_dialog()
         oDlg.run()
@@ -70,7 +69,7 @@ class ClusterCardList(CardListPlugin):
         """Create the dialog for the clustering options."""
         sDlgName = "Cluster Cards in List"
 
-        self.make_prop_groups()
+        self._make_prop_groups()
 
         oDlg = SutekhDialog(sDlgName, self.parent,
                 gtk.DIALOG_DESTROY_WITH_PARENT)
@@ -81,9 +80,9 @@ class ClusterCardList(CardListPlugin):
 
         self._oNotebook = gtk.Notebook()
 
-        oTableSection = self.make_table_section()
-        oAlgorithmSection = self.make_algorithm_section()
-        oResultsSection = self.make_results_section()
+        oTableSection = self._make_table_section()
+        oAlgorithmSection = self._make_algorithm_section()
+        oResultsSection = self._make_results_section()
 
         self._oNotebook.append_page(oTableSection, gtk.Label('Select Columns'))
         self._oNotebook.append_page(oAlgorithmSection, gtk.Label('Settings'))
@@ -96,7 +95,7 @@ class ClusterCardList(CardListPlugin):
 
         return oDlg
 
-    def make_prop_groups(self):
+    def _make_prop_groups(self):
         """Extract the list of possible properties to cluster on."""
         dPropFuncs = CardListTabulator.get_default_prop_funcs()
         self._dGroups = {}
@@ -111,7 +110,7 @@ class ClusterCardList(CardListPlugin):
                         ":".join(aParts[1:]).strip().capitalize()
                 self._dGroups.setdefault(sGroup, {})[sRest] = fProp
 
-    def make_table_section(self):
+    def _make_table_section(self):
         """Create a notebook, and populate the first tabe with a list of
            properties to cluster on."""
         oNotebook = gtk.Notebook()
@@ -140,7 +139,7 @@ class ClusterCardList(CardListPlugin):
                     oHbx.pack_start(oVbx)
 
                 oBut = gtk.CheckButton(sName)
-                oVbx.pack_start(oBut, False) # pack at top, don't fill space
+                oVbx.pack_start(oBut, False)  # pack at top, don't fill space
 
                 self._dPropButtons[sGroup][sName] = oBut
 
@@ -148,7 +147,7 @@ class ClusterCardList(CardListPlugin):
 
         return oNotebook
 
-    def make_algorithm_section(self):
+    def _make_algorithm_section(self):
         """Populate a tab on the notebook with the parameters and options for
            the clustering algorithm.
            """
@@ -157,7 +156,7 @@ class ClusterCardList(CardListPlugin):
         # K-Means is the only clustering solution for the moment
         oHeading = gtk.Label()
         oHeading.set_markup("<b>Parameters for K-Means Clustering</b>")
-        oVbx.pack_start(oHeading, False, False, 5) # top align
+        oVbx.pack_start(oHeading, False, False, 5)  # top align
 
         # Number of iterations
         oNumIterLabel = gtk.Label("Number of Iterations:")
@@ -166,14 +165,14 @@ class ClusterCardList(CardListPlugin):
         self._oNumIterSpin.set_increments(1, 10)
         self._oNumIterSpin.set_value(10)
         oHbox = gtk.HBox(False, 0)
-        oHbox.pack_start(oNumIterLabel, False) # left align
-        oHbox.pack_end(self._oNumIterSpin, False) # right align
+        oHbox.pack_start(oNumIterLabel, False)  # left align
+        oHbox.pack_end(self._oNumIterSpin, False)  # right align
         oVbx.pack_start(oHbox, False)
 
         # Autoset Num clusters
         self._oAutoNumClusters = gtk.CheckButton("One cluster per 80 cards")
         oHbox = gtk.HBox(False, 0)
-        oHbox.pack_start(self._oAutoNumClusters, False) # right align
+        oHbox.pack_start(self._oAutoNumClusters, False)  # right align
         oVbx.pack_start(oHbox, False)
 
         # Number of clusters
@@ -183,8 +182,8 @@ class ClusterCardList(CardListPlugin):
         self._oNumClustersSpin.set_increments(1, 10)
         self._oNumClustersSpin.set_value(10)
         oHbox = gtk.HBox(False, 0)
-        oHbox.pack_start(oNumClustersLabel, False) # left align
-        oHbox.pack_end(self._oNumClustersSpin, False) # right align
+        oHbox.pack_start(oNumClustersLabel, False)  # left align
+        oHbox.pack_end(self._oNumClustersSpin, False)  # right align
         oVbx.pack_start(oHbox, False)
 
         # Connect Autoset and Number of clusters
@@ -222,7 +221,7 @@ class ClusterCardList(CardListPlugin):
 
         return oVbx
 
-    def make_results_section(self):
+    def _make_results_section(self):
         """Create a widget in the notebook which can be used to display
            the clustering results."""
         oVbx = gtk.VBox(False, 0)
@@ -231,7 +230,7 @@ class ClusterCardList(CardListPlugin):
         # Results Heading
         oHeading = gtk.Label()
         oHeading.set_markup("<b>Results</b>")
-        oVbx.pack_start(oHeading, False) # top align
+        oVbx.pack_start(oHeading, False)  # top align
 
         # No Results Yet
         oLabel = gtk.Label("No results yet.")
@@ -244,7 +243,7 @@ class ClusterCardList(CardListPlugin):
     # W0201: We create a lot of attributes here, which is OK, because of
     # plugin structure
     # R0914: We use lots of local variables for clarity
-    def populate_results(self, aCards, aColNames, aMeans, aClusters):
+    def _populate_results(self, aCards, aColNames, aMeans, aClusters):
         """Populate the results tab of the notebook with the results of
            clustering run."""
         self._aCards, self._aColNames, self._aMeans, self._aClusters = \
@@ -258,10 +257,8 @@ class ClusterCardList(CardListPlugin):
         iHeaderRows = 1
         iExtraCols = 4
 
-        oTable = gtk.Table(
-                rows=len(aMeans) + iHeaderRows,
-                columns=iExtraCols
-        )
+        oTable = gtk.Table(rows=len(aMeans) + iHeaderRows,
+                columns=iExtraCols)
         oTable.set_row_spacings(0)
 
         # Headings
@@ -307,7 +304,7 @@ class ClusterCardList(CardListPlugin):
         oMakeCardSetsButton = gtk.Button("Make Card Sets from Selected"
                 " Clusters")
         oMakeCardSetsButton.connect("clicked", self.handle_make_card_sets)
-        self._oResultsVbox.pack_end(oMakeCardSetsButton, False) # bottom align
+        self._oResultsVbox.pack_end(oMakeCardSetsButton, False)  # bottom align
 
         self._oResultsVbox.show_all()
 
@@ -324,27 +321,24 @@ class ClusterCardList(CardListPlugin):
             # change to results section
             self._oNotebook.set_current_page(2)
 
-    # oSomeObj required by function signature
-    def handle_make_card_sets(self, oSomeObj):
+    def handle_make_card_sets(self, _oSomeObj):
         """Create card a suitable card set from the chosen clusters"""
         for iId, oBut in self._dCardSetMakingButtons.iteritems():
             if oBut.get_active():
                 self._fMakeCardSetFromCluster(iId)
 
-    # pylint: enable-msg=W0613
-
     @staticmethod
     def k_means_plus_plus(aCards, iNumClust, fDist):
         """Find a set of initial centers using the k-means++ algorithm.
 
            See http://www.stanford.edu/~darthur/kMeansPlusPlus.pdf.
            """
-        aMeans = [ random.choice(aCards) ]
+        aMeans = [random.choice(aCards)]
 
         while len(aMeans) < iNumClust:
             aDists = []
             for oVec in aCards:
-                fMinD = min((fDist(oVec, oMean) for oMean in aMeans))**2
+                fMinD = min((fDist(oVec, oMean) for oMean in aMeans)) ** 2
                 aDists.append(fMinD)
 
             fSumSq = sum(aDists)
@@ -443,9 +437,9 @@ class ClusterCardList(CardListPlugin):
         aMeans, aClusters = self.k_means(aTable, iNumClusts, iIterations,
                 fDist)
 
-        self.populate_results(aCards, aColNames, aMeans, aClusters)
+        self._populate_results(aCards, aColNames, aMeans, aClusters)
 
-    def make_pcs_from_cluster(self, iClusterId):
+    def _make_pcs_from_cluster(self, iClusterId):
         """Create a Card Set from the chosen cluster"""
         sDeckName = '_cluster_deck_%d' % (iClusterId,)
 
@@ -464,7 +458,7 @@ class ClusterCardList(CardListPlugin):
             oCard = IPhysicalCard(self._aCards[iIndex])
             oDeck.addPhysicalCard(oCard)
 
-        self.open_cs(sDeckName)
+        self.open_cs(sDeckName, True)
 
 
 class Vector(object):
@@ -479,9 +473,9 @@ class Vector(object):
     # pylint: disable-msg=W0212
 
     def euclidian_distance(self, oVec2):
-        """Euclidian distance between two vectors."""
+        """Euclidean distance between two vectors."""
         assert len(self._aData) == len(oVec2._aData)
-        fSum = sum(((x-y)**2 for (x, y) in zip(self._aData, oVec2._aData)))
+        fSum = sum(((x - y) ** 2 for (x, y) in zip(self._aData, oVec2._aData)))
         return math.sqrt(fSum)
 
     METRICS['Euclidean Distance'] = euclidian_distance
@@ -495,7 +489,7 @@ class Vector(object):
         fSum = sum((
             ((x == -1 or y == -1) and 0.25) or
             (((x == 0) ^ (y == 0) and 4.0)) or
-            (x-y)**2 for (x, y) in zip(self._aData, oVec2._aData)
+            (x - y) ** 2 for (x, y) in zip(self._aData, oVec2._aData)
         ))
         return math.sqrt(fSum)
 
@@ -515,24 +509,24 @@ class Vector(object):
         if oOther == 0:
             return self
         assert len(self._aData) == len(oOther._aData)
-        return Vector([ x+y for (x, y) in zip(self._aData, oOther._aData)])
+        return Vector([x + y for (x, y) in zip(self._aData, oOther._aData)])
 
     def __radd__(self, oOther):
         """Sum this vector with another."""
         if oOther == 0:
             return self
         assert len(self._aData) == len(oOther._aData)
-        return Vector([ x+y for (x, y) in zip(self._aData, oOther._aData)])
+        return Vector([x + y for (x, y) in zip(self._aData, oOther._aData)])
 
     def __mul__(self, fScale):
         """Multiply this vector by a scalar."""
         fScale = float(fScale)
-        return Vector([x*fScale for x in self._aData])
+        return Vector([x * fScale for x in self._aData])
 
     def __rmul__(self, fScale):
         """Multiply this vector by a scalar."""
         fScale = float(fScale)
-        return Vector([x*fScale for x in self._aData])
+        return Vector([x * fScale for x in self._aData])
 
     def __len__(self):
         """Length of this vector."""
@@ -546,6 +540,4 @@ class Vector(object):
         """Iterator."""
         return iter(self._aData)
 
-# pylint: disable-msg=C0103
-# accept plugin name
 plugin = ClusterCardList
