@@ -1,34 +1,24 @@
 # JOLDeckParser.py
 # -*- coding: utf8 -*-
 # vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
-# ELDB Deck File Parser
+# JOL Deck File Parser
 # Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
 # GPL - see COPYING for details
 
 """Parser for JOL deck format"""
 
-# pylint: disable-msg=W0402
-# string.digits is OK
 import re
+from sutekh.core.SutekhObjects import csv_to_canonical
+from sutekh.io.IOBase import BaseLineParser
 
-class JOLDeckParser(object):
+
+class JOLDeckParser(BaseLineParser):
     """Parser for the JOL Deck format."""
 
     oCardLineRegexp = re.compile('((?P<num>[0-9]+)(\s)*x(\s)*)?(?P<name>.*)$')
 
-    def __init__(self, oHolder):
-        self._oHolder = oHolder
-
-    def reset(self):
-        """Reset the parser state"""
-        pass
-
-    def feed(self, sLine):
-        """Feed the next line to the parser and extract the cards"""
-        sLine = sLine.strip()
-        if not sLine:
-            # skip blank lines
-            return
+    def _feed(self, sLine, oHolder):
+        """Read the line into the given CardSetHolder"""
         oMatch = self.oCardLineRegexp.match(sLine)
         if oMatch is not None:
             dResults = oMatch.groupdict()
@@ -39,15 +29,11 @@ class JOLDeckParser(object):
             sName = dResults['name']
         else:
             # error condition
-            raise RuntimeError('Unrecognised line for JOL format')
+            raise IOError('Unrecognised line for JOL format')
         # Unescape names
         if sName.endswith('(advanced)'):
             sName = sName.replace('(advanced)', '(Advanced)')
-        # The following two may not be necessary, but it's not unlikely that
-        # people will produce decks with these
-        if sName.endswith(', The'):
-            sName = 'The ' + sName[:-5]
-        elif sName.endswith(', An'):
-            sName = 'An ' + sName[:-4]
+        # Avoid going down the exception path in IAbstractCard if we can
+        sName = csv_to_canonical(sName)
         # JOL has no expansion info
-        self._oHolder.add(iNum, sName, None)
+        oHolder.add(iNum, sName, None)
