@@ -10,10 +10,13 @@
 import codecs
 import urllib2
 
-# pylint: disable-msg=C0103
-# These names are acceptable in this case
-WW_CARDLIST_URL = "http://www.white-wolf.com/vtes/HTML/cardlist.html"
+WW_CARDLIST_URL = "http://www.vekn.net/images/stories/downloads/cardlist.txt"
 
+EXTRA_CARD_URL = "http://bitbucket.org/hodgestar/sutekh-extras/" \
+        "raw/tip/extra_list.txt"
+
+# pylint: disable-msg=C0103
+# We accept the overly long name here
 WW_CARDLIST_PER_LETTER_URLS = [
         "http://www.white-wolf.com/vtes/index.php?line=cardlist__",
         "http://www.white-wolf.com/vtes/index.php?line=cardlist_A",
@@ -44,9 +47,19 @@ WW_CARDLIST_PER_LETTER_URLS = [
         "http://www.white-wolf.com/vtes/index.php?line=cardlist_Z",
         ]
 
-WW_RULINGS_URL = "http://www.white-wolf.com/vtes/index.php?line=rulings"
 # pylint: enable-msg=C0103
 
+WW_OLD_RULINGS_URL = "http://www.white-wolf.com/vtes/index.php?line=rulings"
+
+WW_RULINGS_URL = [
+        # We skip the general rulings, since we only want card rulings
+        #"http://www.vekn.net/index.php/all-rulings/general-rulings",
+        "http://www.vekn.net/index.php/all-rulings/card-rulings-a-to-f",
+        "http://www.vekn.net/index.php/all-rulings/card-rulings-g-to-o",
+        "http://www.vekn.net/index.php/all-rulings/card-rulings-p-to-z",
+        ]
+
+
 class WwFile(object):
     """WwFile is a convenience class which has an .open(..) method which
        returns a file-like object with the encoding set correctly.
@@ -73,4 +86,3 @@ class WwFile(object):
                     'cp1252')
         else:
             return codecs.open(self.sfFile, 'rU', 'cp1252')
-
